PROGRAM FUND28;

(*  Climate Framework for Uncertainty, Negotiation and Distribution       *)
(*  Version 2.80  -  11 February 2004                                     *)
(*  Richard S.J. Tol                                                      *)
(*  Hamburg, Vrije and Carnegie Mellon Universities                       *)

USES
    Crt, RP280, GenRand;

CONST
     scratch = true;                  (* start with no measures ? *)
     CBA = false;                      (* cost-benefit analysis ? *)
     CEA = false;                     (* cost-effectiveness analysis ? *)
     cooperate = false;               (* cooperate ? *)
     repeated = false;
     uncertain = false;               (* uncertainty ? *)
     scengmt = false;                 (* scenario global mean temperature ? *)
     baseincl = false;
     kantian = false;
     compens = false;
     ppp = false;

     consleak = 0.25;                 (* consumption leak *)
     perm = 0.5;                      (* share permanent emission reduction *)
     knowpar = 0.9;
     knowgpar = 0.1;
     compth = 100;                    (* compensation threshold *)

     itmax = 25;

     startper = 1;

     riskave = 1;
     ineqave = 0;

     ampl = 4;
     altruism = 0.00; (* altruism should be zero if cooperation *)

     runs = 0;

     bforst = 1.00;
     btaxst = 0.0001;
     bch4st = 0.0001;
     bn2ost = 0.0001;

     CBaU = 1792.3352;
     CMax = 550;

     gwpch4 = 21*22/6/1000;
     gwpn2o = 310*7/6/1000;

TYPE
    measure = regarr;
    matmeas = ARRAY [1..NoReg,1..NOI] OF double;
    regper = ARRAY [1..NoReg,1..NoPer] OF double;
    boolmea = ARRAY [1..NoReg,1..NOI] OF boolean;

VAR
   forstep, taxstep, ch4step, n2ostep: double;
   c2000, c2025, c2050, c2075, c2100, c2125, c2150, c2175, c2200, c2225, c2250, c2275, c2300: double;
   toptper,globemm,globcum,globknow,minint: double;
   optper: word;
   globviol: boolean;
   emmrec, count: regarr;
   coop: regper;

   spbm, spbmbg, spbmsd: double;
   vsl, vslbg, vslsd, vyll, vyllbg, vyllsd, vmorb, vmorbbg, vmorbsd, valincbg, valincsd, valbase: double;
   watechrate, wrel, wrelbg, wrelsd, wrnl, wrnlbg, wrnlsd: double;
   agel, agelbg, agelsd, agnl, agnlbg, agnlsd, forel, forelbg, forelsd, fornl, fornlbg, fornlsd: double;
   heel, heelbg, heelsd, henl, henlbg, henlsd: double;
   ceel, ceelbg, ceelsd, cenl, cenlbg, cenlsd: double;
   dfnl, dfnlbg, dfnlsd: double;
   smnl, smnlbg, smnlsd: double;
   malnl, malnlbg, malnlsd: double;
   diamortel, diamortelbg, diamortelsd, diayldel, diayldelbg, diayldelsd: double;
   diamortnl, diamortnlbg, diamortnlsd, diayldnl, diayldnlbg, diayldnlsd: double;
   bioloss, biosens, bioshare, nospecies: double;
   dvbmbg, dvbmsd, drydelbg, drydelsd, incdens, wvbmbg, wvbmsd, wetdelbg, wetdelsd: double;
   immcst, immcstbg, immcstsd, entertime, entertimebg, entertimesd: double;
   emcst, emcstbg, emcstsd, leavedel, leavedelbg, leavedelsd: double;
   plusel, pluselbg, pluselsd, cvlin, cvlinbg, cvlinsd, rlin, rlinbg, rlinsd: double;
   maxcardvasc, maxcardvascbg, maxcardvascsd: double;
   addmp, vbel, vbelbg, vbelsd: double;
   forco2, forco2bg,forco2sd: double;
   MC, maxRF, maxyr: double;

FUNCTION Interact(M,N: double): double;
(* interaction radiative forcing methane and nitrous oxide (in ppbv) *)

VAR hulp: double;

BEGIN
     hulp:= 1 + pow(M*N,0.75)*2.01E-5 + pow(M*N,1.52)*M*5.31E-15;
     interact:= 0.47*log(hulp)
END;

PROCEDURE InitImpactPar;

(* CODE
   sp = species
   for = forestry
   ag = agriculture
   he = heating energy
   ce = cooling energy
   wr = water resources
   df = dengue fever
   mal = malaria
   sm = schistosomiasis
   ccp = cardiovascular cold 65 plus
   ccm = cardiovascular cold 65 minus
   chp = cardiovascular heat 65 plus
   chm = cardiovascular heat 65 minus
   rp = respiratory 65 plus
   rm = respiratory 65 minus

   bg = best guess
   sd = standard deviation
   bm = benchmark
   nl = non-linearity
   el = elasticity
   del = delay
   l = linear
   q = quadratic
*)

VAR i,j: word;

BEGIN
     bioloss:= biolossbg;
     biosens:= biosensbg;
     bioshare:= biosharebg;
     wrel:= wrelbg;
     wrnl:= wrnlbg;
     watechrate:= watechratebg;
     agel:= agelbg;
     agnl:= agnlbg;
     forel:= forelbg;
     fornl:= fornlbg;
     malnl:= malnlbg;
     dfnl:= dfnlbg;
     smnl:= smnlbg;
     heel:= heelbg;
     henl:= henlbg;
     ceel:= ceelbg;
     cenl:= cenlbg;
     plusel:= pluselbg;
     vsl:= vslbg;
     vyll:= vyllbg;
     vmorb:= vmorbbg;
     emcst:= emcstbg;
     immcst:= immcstbg;
     cvlin:= cvlinbg;
     rlin:= rlinbg;
     diamortel:= diamortelbg;
     diamortnl:= diamortnlbg;
     spbm:= spbmbg;
     maxcardvasc:= maxcardvascbg;
     leavedel:= leavedelbg;
     entertime:= entertimebg;

     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              valinc:= valincbg;
              valadj:= valbase/valinc/(1+valbase/valinc);
              wrbm:= wrbmbg;
              forbm:= forbmbg;
              hebm:= hebmbg;
              cebm:= cebmbg;
              dfch:= dfchbg;
              smch:= smchbg;
              malch:= malchbg;
              diayldel:= diayldelbg;
              diayldnl:= diayldnlbg;
              agrbm:= agrbmbg;
              aglbm:= aglbmbg;
              agopt:= agoptbg;
              aglparl:= -2*agopt*aglbm/(10.24-6.4*agopt);
              aglparq:= aglbm/(10.24-6.4*agopt);
              IF aglparq > 0 THEN BEGIN
                 aglparq:= 0;
                 aglparl:= aglbm/3.2
              END;
              agcbm:= agcbmbg/ln(2);
              agtime:= agtimebg;
              ccplbm:= ccplbmbg;
              ccpqbm:= ccpqbmbg;
              ccmlbm:= ccmlbmbg;
              ccmqbm:= ccmqbmbg;
              chplbm:= chplbmbg;
              chpqbm:= chpqbmbg;
              chmlbm:= chmlbmbg;
              chmqbm:= chmqbmbg;
              vbel:= vbelbg;
              rlbm:= rlbmbg;
              rqbm:= rqbmbg;
              wlbm:= wlbmbg;
              wvbm:= wvbmbg;
              wetdel:= wetdelbg;
              wmbm:= wmbmbg;
              dlbm:= dlbmbg;
              dvbm:= dvbmbg;
              drydel:= drydelbg;
              pc:= pcbg;
              FOR j:= 1 TO NoReg DO
                  immigrate[j]:= immigratebg[j]
         END
END;

PROCEDURE InitCoalition;

VAR i,j: word;

BEGIN
       coop[1,1]:= 0;   coop[1,2]:= 0;   coop[1,3]:= 0;   coop[1,4]:= 0;
       coop[1,5]:= 0;   coop[1,6]:= 0;   coop[1,7]:= 0;   coop[1,8]:= 0;
       coop[1,9]:= 0;  coop[1,10]:= 0;  coop[1,11]:= 0;  coop[1,12]:= 0;
      coop[1,13]:= 0;  coop[1,14]:= 0;  coop[1,15]:= 0;  coop[1,16]:= 0;
      coop[1,17]:= 0;  coop[1,18]:= 0;

       coop[2,1]:= 0;   coop[2,2]:= 0;   coop[2,3]:= 0;   coop[2,4]:= 0;
       coop[2,5]:= 1;   coop[2,6]:= 1;   coop[2,7]:= 1;   coop[2,8]:= 1;
       coop[2,9]:= 1;  coop[2,10]:= 1;  coop[2,11]:= 1;  coop[2,12]:= 1;
      coop[2,13]:= 1;  coop[2,14]:= 1;  coop[2,15]:= 1;  coop[2,16]:= 1;
      coop[2,17]:= 1;  coop[2,18]:= 1;

       coop[3,1]:= 0;   coop[3,2]:= 0;   coop[3,3]:= 0;   coop[3,4]:= 0;
       coop[3,5]:= 0;   coop[3,6]:= 0;   coop[3,7]:= 0;   coop[3,8]:= 0;
       coop[3,9]:= 0;  coop[3,10]:= 0;  coop[3,11]:= 0;  coop[3,12]:= 0;
      coop[3,13]:= 0;  coop[3,14]:= 0;  coop[3,15]:= 0;  coop[3,16]:= 0;
      coop[3,17]:= 0;  coop[3,18]:= 0;

       coop[4,1]:= 1;   coop[4,2]:= 1;   coop[4,3]:= 1;   coop[4,4]:= 1;
       coop[4,5]:= 1;   coop[4,6]:= 1;   coop[4,7]:= 1;   coop[4,8]:= 1;
       coop[4,9]:= 1;  coop[4,10]:= 1;  coop[4,11]:= 1;  coop[4,12]:= 1;
      coop[4,13]:= 1;  coop[4,14]:= 1;  coop[4,15]:= 1;  coop[4,16]:= 1;
      coop[4,17]:= 1;  coop[4,18]:= 1;

       coop[5,1]:= 0;   coop[5,2]:= 0;   coop[5,3]:= 0;   coop[5,4]:= 0;
       coop[5,5]:= 0;   coop[5,6]:= 0;   coop[5,7]:= 0;   coop[5,8]:= 0;
       coop[5,9]:= 0;  coop[5,10]:= 0;  coop[5,11]:= 0;  coop[5,12]:= 0;
      coop[5,13]:= 0;  coop[5,14]:= 0;  coop[5,15]:= 0;  coop[5,16]:= 0;
      coop[5,17]:= 0;  coop[5,18]:= 0;

       coop[6,1]:= 0;   coop[6,2]:= 0;   coop[6,3]:= 0;   coop[6,4]:= 0;
       coop[6,5]:= 0;   coop[6,6]:= 0;   coop[6,7]:= 0;   coop[6,8]:= 0;
       coop[6,9]:= 1;  coop[6,10]:= 1;  coop[6,11]:= 1;  coop[6,12]:= 1;
      coop[6,13]:= 1;  coop[6,14]:= 1;  coop[6,15]:= 1;  coop[6,16]:= 1;
      coop[6,17]:= 1;  coop[6,18]:= 1;

       coop[7,1]:= 0;   coop[7,2]:= 0;   coop[7,3]:= 0;   coop[7,4]:= 0;
       coop[7,5]:= 0;   coop[7,6]:= 0;   coop[7,7]:= 1;   coop[7,8]:= 1;
       coop[7,9]:= 1;  coop[7,10]:= 1;  coop[7,11]:= 1;  coop[7,12]:= 1;
      coop[7,13]:= 1;  coop[7,14]:= 1;  coop[7,15]:= 1;  coop[7,16]:= 1;
      coop[7,17]:= 1;  coop[7,18]:= 1;

       coop[8,1]:= 0;   coop[8,2]:= 0;   coop[8,3]:= 0;   coop[8,4]:= 0;
       coop[8,5]:= 0;   coop[8,6]:= 0;   coop[8,7]:= 0;   coop[8,8]:= 0;
       coop[8,9]:= 0;  coop[8,10]:= 0;  coop[8,11]:= 0;  coop[8,12]:= 0;
      coop[8,13]:= 0;  coop[8,14]:= 0;  coop[8,15]:= 0;  coop[8,16]:= 0;
      coop[8,17]:= 0;  coop[8,18]:= 0;

       coop[9,1]:= 1;   coop[9,2]:= 1;   coop[9,3]:= 1;   coop[9,4]:= 1;
       coop[9,5]:= 1;   coop[9,6]:= 1;   coop[9,7]:= 1;   coop[9,8]:= 1;
       coop[9,9]:= 1;  coop[9,10]:= 1;  coop[9,11]:= 1;  coop[9,12]:= 1;
      coop[9,13]:= 1;  coop[9,14]:= 1;  coop[9,15]:= 1;  coop[9,16]:= 1;
      coop[9,17]:= 1;  coop[9,18]:= 1;

      coop[10,1]:= 0;  coop[10,2]:= 0;  coop[10,3]:= 0;  coop[10,4]:= 0;
      coop[10,5]:= 0;  coop[10,6]:= 0;  coop[10,7]:= 0;  coop[10,8]:= 0;
      coop[10,9]:= 0; coop[10,10]:= 0; coop[10,11]:= 0; coop[10,12]:= 0;
     coop[10,13]:= 0; coop[10,14]:= 0; coop[10,15]:= 0; coop[10,16]:= 0;
     coop[10,17]:= 0; coop[10,18]:= 0;

      coop[11,1]:= 1;  coop[11,2]:= 1;  coop[11,3]:= 1;  coop[11,4]:= 1;
      coop[11,5]:= 1;  coop[11,6]:= 1;  coop[11,7]:= 1;  coop[11,8]:= 1;
      coop[11,9]:= 1; coop[11,10]:= 1; coop[11,11]:= 1; coop[11,12]:= 1;
     coop[11,13]:= 1; coop[11,14]:= 1; coop[11,15]:= 1; coop[11,16]:= 1;
     coop[11,17]:= 1; coop[11,18]:= 1;

      coop[12,1]:= 0;  coop[12,2]:= 0;  coop[12,3]:= 0;  coop[12,4]:= 0;
      coop[12,5]:= 0;  coop[12,6]:= 0;  coop[12,7]:= 0;  coop[12,8]:= 0;
      coop[12,9]:= 0; coop[12,10]:= 0; coop[12,11]:= 0; coop[12,12]:= 0;
     coop[12,13]:= 0; coop[12,14]:= 0; coop[12,15]:= 0; coop[12,16]:= 0;
     coop[12,17]:= 0; coop[12,18]:= 0;

      coop[13,1]:= 0;  coop[13,2]:= 0;  coop[13,3]:= 0;  coop[13,4]:= 0;
      coop[13,5]:= 0;  coop[13,6]:= 0;  coop[13,7]:= 0;  coop[13,8]:= 0;
      coop[13,9]:= 1; coop[13,10]:= 1; coop[13,11]:= 1; coop[13,12]:= 1;
     coop[13,13]:= 1; coop[13,14]:= 1; coop[13,15]:= 1; coop[13,16]:= 1;
     coop[13,17]:= 1; coop[13,18]:= 1;

      coop[14,1]:= 0;  coop[14,2]:= 0;  coop[14,3]:= 0;  coop[14,4]:= 0;
      coop[14,5]:= 0;  coop[14,6]:= 0;  coop[14,7]:= 1;  coop[14,8]:= 1;
      coop[14,9]:= 1; coop[14,10]:= 1; coop[14,11]:= 1; coop[14,12]:= 1;
     coop[14,13]:= 1; coop[14,14]:= 1; coop[14,15]:= 1; coop[14,16]:= 1;
     coop[14,17]:= 1; coop[14,18]:= 1;

      coop[15,1]:= 0;  coop[15,2]:= 0;  coop[15,3]:= 0;  coop[15,4]:= 0;
      coop[15,5]:= 0;  coop[15,6]:= 0;  coop[15,7]:= 0;  coop[15,8]:= 0;
      coop[15,9]:= 0; coop[15,10]:= 0; coop[15,11]:= 0; coop[15,12]:= 0;
     coop[15,13]:= 0; coop[15,14]:= 0; coop[15,15]:= 0; coop[15,16]:= 0;
     coop[15,17]:= 0; coop[15,18]:= 0;

      coop[16,1]:= 1;  coop[16,2]:= 1;  coop[16,3]:= 1;  coop[16,4]:= 1;
      coop[16,5]:= 1;  coop[16,6]:= 1;  coop[16,7]:= 1;  coop[16,8]:= 1;
      coop[16,9]:= 1; coop[16,10]:= 1; coop[16,11]:= 1; coop[16,12]:= 1;
     coop[16,13]:= 1; coop[16,14]:= 1; coop[16,15]:= 1; coop[16,16]:= 1;
     coop[16,17]:= 1; coop[16,18]:= 1;

     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             coop[i,j]:= 0
END;

PROCEDURE Initialise;

VAR i, j: word;
    fil: text;

BEGIN
     addmp:= 1;
     optper:= startper;
     C2200:= CBaU;
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              oda:= 0;
              new(ypcgrowth);
              new(pgrowth);
              new(aeei);
              new(acei);
              new(ch4em);
              new(n2oem);
              new(forestemm);
              new(scentemp)
         END;
     WITH Climate DO BEGIN
          decay[1]:= 1;
          decay[2]:= exp(-1/lifeco2);
          decay[3]:= exp(-1/lifeco3);
          decay[4]:= exp(-1/lifeco4);
          decay[5]:= exp(-1/lifeco5);
          frac[1]:= 0.13;
          frac[2]:= 0.20;
          frac[3]:= 0.32;
          frac[4]:= 0.25;
          frac[5]:= 0.10;
          ch4decay:= ch4dec;
          n2odecay:= n2odec;
          ch4sd:= 6.8346;
          n2osd:= 0.0059;
          new(addco2);
          new(addch4);
          new(addn2o);
          new(acco2);
          new(acch4);
          new(acn2o)
     END;

     Economy[1].name:=  'USA';     Economy[2].name:=  'CAN';
     Economy[3].name:=  'WEU';     Economy[4].name:=  'JPK';
     Economy[5].name:=  'ANZ';     Economy[6].name:=  'EEU';
     Economy[7].name:=  'FSU';     Economy[8].name:=  'MDE';
     Economy[9].name:=  'CAM';     Economy[10].name:= 'LAM';
     Economy[11].name:= 'SAS';     Economy[12].name:= 'SEA';
     Economy[13].name:= 'CHI';     Economy[14].name:= 'MAF';
     Economy[15].name:= 'SSA';     Economy[16].name:= 'SIS';

     Economy[1].GDP90:=  6343.0;   Economy[2].GDP90:=   532.5;
     Economy[3].GDP90:=  8285.4;   Economy[4].GDP90:=  5100.7;
     Economy[5].GDP90:=   359.5;   Economy[6].GDP90:=   338.1;
     Economy[7].GDP90:=   915.1;   Economy[8].GDP90:=   447.3;
     Economy[9].GDP90:=   296.0;   Economy[10].GDP90:= 1078.0;
     Economy[11].GDP90:=  351.5;   Economy[12].GDP90:=  456.2;
     Economy[13].GDP90:=  511.4;   Economy[14].GDP90:=  137.3;
     Economy[15].GDP90:=  275.1;   Economy[16].GDP90:=   37.2;

     Economy[1].pop90:=   254.1;   Economy[2].pop90:=    27.8;
     Economy[3].pop90:=   376.6;   Economy[4].pop90:=   166.4;
     Economy[5].pop90:=    20.2;   Economy[6].pop90:=   124.3;
     Economy[7].pop90:=   289.6;   Economy[8].pop90:=   189.2;
     Economy[9].pop90:=   106.3;   Economy[10].pop90:=  300.2;
     Economy[11].pop90:= 1131.7;   Economy[12].pop90:=  444.1;
     Economy[13].pop90:= 1184.1;   Economy[14].pop90:=  117.8;
     Economy[15].pop90:=  494.4;   Economy[16].pop90:=   39.9;

     Economy[1].urbcorr:=   0.1163;   Economy[2].urbcorr:=   0.0659;
     Economy[3].urbcorr:=   0.1578;   Economy[4].urbcorr:=   0.1558;
     Economy[5].urbcorr:=  -0.0516;   Economy[6].urbcorr:=   0.0051;
     Economy[7].urbcorr:=  -0.0121;   Economy[8].urbcorr:=  -0.0391;
     Economy[9].urbcorr:=  -0.0892;   Economy[10].urbcorr:= -0.1592;
     Economy[11].urbcorr:=  0.3132;   Economy[12].urbcorr:=  0.8026;
     Economy[13].urbcorr:=  0.6088;   Economy[14].urbcorr:=  0.0611;
     Economy[15].urbcorr:=  0.3879;   Economy[16].urbcorr:=  0.0095;

     Economy[1].coastpd:=   1.04;     Economy[2].coastpd:=   1.07;
     Economy[3].coastpd:=   4.48;     Economy[4].coastpd:=   1.04;
     Economy[5].coastpd:=   1.27;     Economy[6].coastpd:=   3.01;
     Economy[7].coastpd:=   0.75;     Economy[8].coastpd:=   3.42;
     Economy[9].coastpd:=   0.69;     Economy[10].coastpd:=  1.97;
     Economy[11].coastpd:=  2.31;     Economy[12].coastpd:=  1.80;
     Economy[13].coastpd:=  4.46;     Economy[14].coastpd:= 17.61;
     Economy[15].coastpd:=  9.69;     Economy[16].coastpd:=  2.51;

     FOR i:= 1 TO NoReg DO
         Economy[i].ypc90:= Economy[i].GDP90/Economy[i].pop90*1000;

     Economy[1].plus90:=  0.1239;     Economy[2].plus90:=  0.1122;
     Economy[3].plus90:=  0.1473;     Economy[4].plus90:=  0.1490;
     Economy[5].plus90:=  0.1115;     Economy[6].plus90:=  0.1034;
     Economy[7].plus90:=  0.0931;     Economy[8].plus90:=  0.0358;
     Economy[9].plus90:=  0.0384;     Economy[10].plus90:= 0.0486;
     Economy[11].plus90:= 0.0406;     Economy[12].plus90:= 0.0400;
     Economy[13].plus90:= 0.0856;     Economy[14].plus90:= 0.0373;
     Economy[15].plus90:= 0.0291;     Economy[16].plus90:= 0.0604;

     Economy[1].cardvasc90:=  0.00350;     Economy[2].cardvasc90:=  0.00350;
     Economy[3].cardvasc90:=  0.00436;     Economy[4].cardvasc90:=  0.00263;
     Economy[5].cardvasc90:=  0.00263;     Economy[6].cardvasc90:=  0.00481;
     Economy[7].cardvasc90:=  0.00874;     Economy[8].cardvasc90:=  0.00198;
     Economy[9].cardvasc90:=  0.00182;     Economy[10].cardvasc90:= 0.00182;
     Economy[11].cardvasc90:= 0.00281;     Economy[12].cardvasc90:= 0.00203;
     Economy[13].cardvasc90:= 0.00205;     Economy[14].cardvasc90:= 0.00237;
     Economy[15].cardvasc90:= 0.00149;     Economy[16].cardvasc90:= 0.00182;

     Economy[1].resp90:=  0.00053;     Economy[2].resp90:=  0.00053;
     Economy[3].resp90:=  0.00050;     Economy[4].resp90:=  0.00037;
     Economy[5].resp90:=  0.00037;     Economy[6].resp90:=  0.00030;
     Economy[7].resp90:=  0.00052;     Economy[8].resp90:=  0.00019;
     Economy[9].resp90:=  0.00039;     Economy[10].resp90:= 0.00039;
     Economy[11].resp90:= 0.00039;     Economy[12].resp90:= 0.00046;
     Economy[13].resp90:= 0.00011;     Economy[14].resp90:= 0.00037;
     Economy[15].resp90:= 0.00038;     Economy[16].resp90:= 0.00039;

     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             economy[i].trade[j]:= 0;

     ReadWater;
     ReadForestry;
     ReadHeating;
     ReadCooling;
     ReadDengue;
     ReadSchisto;
     ReadMalaria;
     ReadDiarrhea;
     ReadAgricult;
     ReadCardHeat;
     ReadCardCold;
     ReadResp;
     ReadYLL;
     ReadYLD;
     ReadWetland;
     ReadDryland;
     ReadProtect;
     ReadImmigrate;

     ReadIncome;
     ReadPopulation;
     ReadYpCGrowth;
     ReadAgrish;
     ReadPGrowth;
     ReadACEI;
     ReadAEEI;
     ReadForestEmm;
     ReadForParam;
     ReadDiscRate;
     ReadAddCO2;
     ReadAddCH4;
     ReadAddN2O;
     ReadCH4Em;
     ReadN2OEm;
     ReadBaseRegTemp;
     ReadScenGMT;
     ReadNCEmmRestr;
     ReadCEmmRestr;
     ReadScenUnc;
     ReadMethPar;
     ReadNitrOxPar;
     WITH climate DO BEGIN
          temps:= modet/6.3/ln(2);
          seas:= modes;
          delaysea:= 1/lifesea;
          delaytemp:= 1/lifetemp
     END;
     forstep:= bforst;
     taxstep:= btaxst;
     ch4step:= bch4st;
     n2ostep:= bn2ost;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              ecgradd:= 0;
              pgadd:= 0;
              aeeiadd:= 0;
              aceiadd:= 0;
              foremadd:= 0;
              taxmp:= 1;
              formp:= 1
         END;
     valbase:= 24401;
     valincbg:= 30000;
     valincsd:= 10000;
  (* 20000 is benchmark income; 15965.86 is OECD income in 1990 *)
     spbmbg:= 50;
     spbmsd:= 50;
     dvbmbg:= 4;
     dvbmsd:= 2;
     drydelbg:= 0;
     drydelsd:= 0.1;
     incdens:= 635;
     (* 635 is income density in 1990 in OECD *)
     wvbmbg:= 5;
     wvbmsd:= 2.5;
     wetdelbg:= 0;
     wetdelsd:= 0.1;
     immcstbg:= 0.4;
     immcstsd:= 0.2;
     entertimebg:= 3;
     entertimesd:= 1;
     emcstbg:= 3;
     emcstsd:= 1.5;
     leavedelbg:= 0;
     leavedelsd:= 0.1;
     wrelbg:= 0.85;
     wrelsd:= 0.15;
     wrnlbg:= 1;
     wrnlsd:= 0.5;
     forelbg:= 0.31;
     forelsd:= 0.20;
     fornlbg:= 1.0;
     fornlsd:= 0.5;
     forco2bg:=2.2972;
     forco2sd:=2.2972/2;
     heelbg:= 0.8;
     heelsd:= 0.2;
     henlbg:= 0.5;
     henlsd:= 0.5;
     ceelbg:= 0.8;
     ceelsd:= 0.2;
     cenlbg:= 1.5;
     cenlsd:= 0.5;
     dfnlbg:= 1.0;
     dfnlsd:= 0.5;
     smnlbg:= 1.0;
     smnlsd:= 0.5;
     malnlbg:= 1.0;
     malnlsd:= 0.5;
     diayldelbg:= -0.418406;
     diayldelsd:= 0.119223;
     diamortelbg:= -1.578625;
     diamortelsd:= 0.232507;
     diayldnlbg:= 0.699241;
     diayldnlsd:= 0.260643;
     diamortnlbg:= 1.141714;
     diamortnlsd:= 0.508302;
     agnlbg:= 2;
     agnlsd:= 0.5;
     agelbg:= 0.31;
     agelsd:= 0.15;
     pluselbg:= 0.25;
     pluselsd:= 0.08;
     vbelbg:= -2.65;
     vbelsd:= 0.69;
     IF ppp THEN BEGIN
        agelbg:= agelbg*0.7195;
        pluselbg:= pluselbg*0.7195;
        vbelbg:= -3.26;
        vbelsd:=1.05
     END;
     cvlinbg:= 0.025901;
     cvlinsd:= 0.009574;
     rlinbg:= 0.001583;
     rlinsd:= 0.000457;
     maxcardvascbg:= 0.05;
     maxcardvascsd:= 0.02;
     vslbg:= 200;
     vslsd:= 100;
     vyllbg:= 10;
     vyllsd:= 5;
     vmorbbg:= 0.8;
     vmorbsd:= 1.2
END;

PROCEDURE ReInit;

VAR i, j: word;

BEGIN
     ReadMethPar; (* ??? *)
     ReadNitroxPar; (* ??? *)
     economy[1].emissint:= 1.315*0.273;  economy[2].emissint:= 1.557*0.273;
     economy[3].emissint:= 0.645*0.273;  economy[4].emissint:= 0.121*0.273;
     economy[5].emissint:= 0.597*0.273;  economy[6].emissint:= 2.678*0.273;
     economy[7].emissint:= 3.324*0.273;  economy[8].emissint:= 0.381*0.273;
     economy[9].emissint:= 0.878*0.273;  economy[10].emissint:= 0.648*0.273;
     economy[11].emissint:= 0.839*0.273; economy[12].emissint:= 0.437*0.273;
     economy[13].emissint:= 2.226*0.273; economy[14].emissint:= 1.174*0.273;
     economy[15].emissint:= 0.796*0.273; economy[16].emissint:= 1.733*0.273;

     economy[1].cummemm:=  22967;        economy[2].cummemm:=   1999;
     economy[3].cummemm:=  25854;        economy[4].cummemm:=   1135;
     economy[5].cummemm:=    526;        economy[6].cummemm:=   2199;
     economy[7].cummemm:=   3214;        economy[8].cummemm:=     86;
     economy[9].cummemm:=    212;        economy[10].cummemm:=   696;
     economy[11].cummemm:=   569;        economy[12].cummemm:=   225;
     economy[13].cummemm:=   775;        economy[14].cummemm:=    64;
     economy[15].cummemm:=   472;        economy[16].cummemm:=     0;

     Economy[1].area:=   9207581;       Economy[2].area:=   9872483;
     Economy[3].area:=   3705040;       Economy[4].area:=    472934;
     Economy[5].area:=   9235211;       Economy[6].area:=   2186128;
     Economy[7].area:=  21716915;       Economy[8].area:=   6313671;
     Economy[9].area:=   2440575;       Economy[10].area:= 17451759;
     Economy[11].area:=  5137567;       Economy[12].area:=  4958546;
     Economy[13].area:= 10958008;       Economy[14].area:=  3953026;
     Economy[15].area:= 24329621;       Economy[16].area:=   332534;

     Economy[1].so2:=   1.9884;       Economy[2].so2:=   0.1529;
     Economy[3].so2:=   2.8172;       Economy[4].so2:=   0.1562;
     Economy[5].so2:=   0.0171;       Economy[6].so2:=   0.4115;
     Economy[7].so2:=   2.1540;       Economy[8].so2:=   0.0421;
     Economy[9].so2:=   0.0874;       Economy[10].so2:=  0.3172;
     Economy[11].so2:=  0.7243;       Economy[12].so2:=  0.4571;
     Economy[13].so2:=  4.9740;       Economy[14].so2:=  0.0856;
     Economy[15].so2:=  0.2057;       Economy[16].so2:=  0.0255;

     globcum:= 60994.3;
     globknow:= 1;
     nospecies:= nospecbase;

     minint:= 1000;
     WITH climate DO BEGIN
          biodiv:= 1;
          ch4add:= 0;
          n2oadd:= 0;
          temp:= 0.20;
          dt:= 0.004986;
          sea:= 0;
          ds:= 0.001255;
          cbox[1]:= 283.53;
          cbox[2]:= 5.62;
          cbox[3]:= 6.29;
          cbox[4]:= 2.19;
          cbox[5]:= 0.15;
          acco2^[0]:= 0;
          acch4^[0]:= 1222;
          acn2o^[0]:= 296;
          FOR i:= 1 TO 5 DO
              acco2^[0]:= acco2^[0] + cbox[i];
          ch4n2o:= interact(ch4pre,n2opre)
     END;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              decade:= 0;
              kant:= 0;
              income:= GDP0;
              population:= pop0;
              popdens:= population/area*1000000;
              IF riskave = 1 THEN
                 welfare:= log(1000*(1-consleak)*GDP0/pop0)
              ELSE
                  welfare:= pow(1000*(1-consleak)*GDP0/pop0,1-riskave)/(1-riskave);
              YpC:= 1000*GDP0/pop0;                     (* ????????? *)
              urbpop:= (0.031*root(ypc)-0.011*root(popdens))/(1+0.031*root(ypc)-0.011*root(popdens));
              energint:= 1;                             (* confirmed *)
              energuse:= GDP0;
              ryg:= 0;
              ch4cost:= 0;
              n2ocost:= 0;
              reei:= 0;                                 (* confirmed *)
              rcei:= 0;                                 (* confirmed *)
              seei:= 0;
              scei:= 0;
              htax:= 0;
              know:= 1;
              emission:= emissint*energuse;
              emiss0:= emission;                        (* confirmed *)
              IF emiss0/GDP0 < minint THEN
                 minint:= emiss0/GDP0;
              cdr:= 1;

              agrate:= agrbm*pow(0.005/dbst,agnl)*agtime;
              wetmeas:= 0;
              wettot:= 0;
              dryland:= 0;
              drycost:= 0;
              wetcost:= 0;
              leave:= 0;
              cumleave:= 0;
              cumenter:= 0;
              entercost:= 0;
              leavecost:= 0;
              watech:= 1;
              cumaeei:= 1;
              regtmp:= bregtmp*climate.temp;
              regdt:= bregtmp*climate.dt;
              eloss:= 0;
              sloss:= 0
     END;
END;

(* TAX PROCEDURES *)
(*
PROCEDURE SetTax1(i,j: word);
(* sets tax country I in decade J to zero *)
(*
BEGIN
     economy[i].tax[j]:= 0;
END;
*)
PROCEDURE SetTax2;
(* sets all taxes to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             economy[i].tax[j]:= 0;
END;
(*
PROCEDURE SetTax3(i: word; T: double);

BEGIN
     economy[i].tax[optper]:= T
END;
*)
PROCEDURE SetTax4(T: measure);

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         economy[i].tax[optper]:= T[i]
END;
(*
PROCEDURE SetTax5(i,j: word; T: double);

BEGIN
     economy[i].tax[j]:= T
END;
*)
PROCEDURE ObjTax(i: word);
(* changes taxes and influences for economy I *)

BEGIN
     economy[i].tax[optper]:= economy[i].tax[optper] + taxstep
END;

PROCEDURE TaxObj(i: word);
(* changes taxes and influences one taxstep back for economy I *)

BEGIN
     economy[i].tax[optper]:= economy[i].tax[optper] - taxstep
END;

(* METHANE PROCEDURES *)
(*
PROCEDURE SetCH41(i,j: word);
(* sets methane reduction in country I in decade J to zero *)
(*
BEGIN
     economy[i].ch4red[j]:= 0;
END;
*)
PROCEDURE SetCH42;
(* sets all methane reduction to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             economy[i].ch4red[j]:= 0;
END;
(*
PROCEDURE SetCH43(i: word; T: double);

BEGIN
     economy[i].ch4red[optper]:= T;
END;
*)
PROCEDURE SetCH44(T: measure);

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         economy[i].CH4red[optper]:= T[i]
END;
(*
PROCEDURE SetCH45(i,j: word; T: double);

BEGIN
     economy[i].CH4red[j]:= T
END;
*)
PROCEDURE ObjCH4(i: word);
(* changes methane reduction and influences for economy I *)

BEGIN
     economy[i].ch4red[optper]:= economy[i].ch4red[optper] + ch4step
END;

PROCEDURE CH4Obj(i: word);
(* changes taxes and influences one taxstep back for economy I *)

BEGIN
     economy[i].ch4red[optper]:= economy[i].ch4red[optper] - ch4step
END;

(* NITROUS OXIDE PROCEDURES *)
(*
PROCEDURE SetN2O1(i,j: word);
(* sets nitrous oxide reduction in country I in decade J to zero *)
(*
BEGIN
     economy[i].n2ored[j]:= 0;
END;
*)
PROCEDURE SetN2O2;
(* sets all nitrous oxide reduction to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             economy[i].n2ored[j]:= 0;
END;
(*
PROCEDURE SetN2O3(i: word; T: double);

BEGIN
     economy[i].n2ored[optper]:= T;
END;
*)
PROCEDURE SetN2O4(T: measure);

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         economy[i].n2ored[optper]:= T[i]
END;
(*
PROCEDURE SetN2O5(i,j: word; T: double);

BEGIN
     economy[i].n2ored[j]:= T
END;
*)
PROCEDURE ObjN2O(i: word);
(* changes methane reduction and influences for economy I *)

BEGIN
     economy[i].n2ored[optper]:= economy[i].n2ored[optper] + n2ostep
END;

PROCEDURE N2OObj(i: word);
(* changes taxes and influences one taxstep back for economy I *)

BEGIN
     economy[i].n2ored[optper]:= economy[i].n2ored[optper] - n2ostep
END;

(* FORESTRY PROCEDURES *)

PROCEDURE SetInflFor(VAR country: regrec; t: word);
(* determines the influence of COUNTRY.FORESTRY on the economy of COUNTRY *)

VAR fca, fcd, fa, fd, th1, th2: double;

BEGIN
     WITH country DO BEGIN
          IF forestry[optper] > 0 THEN BEGIN
             th1:= forconst - forlinear + root(quad(forconst-forlinear)-4*forconst*forsquare);
             th1:= th1/2/forsquare;
             th2:= (3*forconst/2 - forlinear)/2/forsquare;
             IF forestry[optper] < (th1 + th2) THEN BEGIN
                fd:= forestry[optper];
                fa:= 0
             END
             ELSE IF forestry[optper] < (2*th1 + th2) THEN BEGIN
                  fd:= th1 + th2;
                  fa:= forestry[optper] - fd
             END
             ELSE BEGIN
                  fa:= th1;
                  fd:= forestry[optper] - fa
             END;
             IF fd > forestemm^[t] THEN BEGIN    (* !!! *)
                fd:= forestemm^[t];
                fa:= forestry[optper] - fd
             END;
             IF fd > 0 THEN BEGIN
                fcd:= 2*(forconst + forlinear*fd + forsquare*quad(fd))/3;
                IF fcd < forconst*fd THEN
                   fcd:= 2*forconst*fd/3
             END
             ELSE
                 fcd:= 0;
             IF fa > 0 THEN BEGIN
                fca:= forconst + forlinear*fa + forsquare*quad(fa);
                IF fca < forconst*fa THEN
                   fca:= forconst*fa
             END
             ELSE
                 fca:= 0;
             forcost:= fca + fcd
          END
          ELSE
              forcost:= 0
     END
END;
(*
PROCEDURE SetFor1(i,j: word);
(* sets forestry in country I in decade J to zero *)
(*
BEGIN
     economy[i].forestry[j]:= 0;
END;
*)
PROCEDURE SetFor2;
(* sets all forestry to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO NoPer DO
             economy[i].forestry[j]:= 0;
END;
(*
PROCEDURE SetFor3(i: word; T: double);

BEGIN
     economy[i].forestry[optper]:= T;
END;
*)
PROCEDURE SetFor4(T: measure);

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         economy[i].forestry[optper]:= T[i]
END;

PROCEDURE ObjFor(i: word);
(* changes forestry in economy I *)

BEGIN
     economy[i].forestry[optper]:= economy[i].forestry[optper] + forstep
END;

PROCEDURE ForObj(i: word);
(* changes forestry one forstep back for economy I *)

BEGIN
     economy[i].forestry[optper]:= economy[i].forestry[optper] - forstep
END;

(* SETTINGS *)

PROCEDURE SetMeasures1;
(* sets all measures to zero *)

BEGIN
     SetTax2;
     SetFor2;
     SetN2O2;
     SetCH42
END;
(*
PROCEDURE SetMeasures2;

VAR i,j: word;

BEGIN
     FOR i:= 1 TO NoPer DO
         FOR j:= 1 TO NoReg DO
             economy[j].tax[i]:= random/100
END;
*)
PROCEDURE SetMeasures3(w: word);

VAR s: string;
    inp: text;
    i,j: word;

BEGIN
     str(w,s);
     assign(inp,sessionpathname+'seq'+s+'.out');
     reset(inp);
     FOR i:= 0 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].tax[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].ch4red[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].n2ored[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].forestry[i]);
         readln(inp)
     END;
     close(inp)
END;

PROCEDURE SetMeasures4;

VAR inp: text;
    i,j: word;

BEGIN
     assign(inp,sessionpathname+'cea.out');
     reset(inp);
     FOR i:= 1 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].tax[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].ch4red[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].n2ored[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].forestry[i]);
         readln(inp)
     END;
     close(inp)
END;

FUNCTION CheckStop(w: word): boolean;

VAR s: string;
    inp0,inp1: text;
    i,j: word;
    h0,h1: double;
    stop: boolean;

BEGIN
     str(w-1,s);
     assign(inp0,sessionpathname+'seq'+s+'.out');
     reset(inp0);
     str(w,s);
     assign(inp1,sessionpathname+'seq'+s+'.out');
     reset(inp1);
     stop:= true;
     FOR i:= 0 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO BEGIN
             read(inp0,h0);
             read(inp1,h1);
             stop:= stop AND (ABS(h1-h0) < 0.5*btaxst)
         END;
         readln(inp0);
         readln(inp1);
         FOR j:= 1 TO NoReg DO BEGIN
             read(inp0,h0);
             read(inp1,h1);
             stop:= stop AND (ABS(h1-h0) < 0.5*bch4st)
         END;
         readln(inp0);
         readln(inp1);
         FOR j:= 1 TO NoReg DO BEGIN
             read(inp0,h0);
             read(inp1,h1);
             stop:= stop AND (ABS(h1-h0) < 0.5*bforst)
         END;
         readln(inp0);
         readln(inp1)
     END;
     close(inp0);
     close(inp1);
     checkstop:= stop
END;

(* CALCULATING PROCEDURES *)

PROCEDURE Calculate(VAR country: regrec; t: word);

VAR i, j, decade: word;
    s, h, henter, excessloss, popold, ic, dc, x, noemer, teller, seef, time: double;
    ygrowth: double;
    incth: boolean;

BEGIN
     time:= t;
     WITH country, climate DO BEGIN
          IF t > 50 THEN BEGIN
             ygrowth:= (1+0.01*(pgrowth^[t-1]+pgadd*(t-50)))*(1+0.01*(ypcgrowth^[t-1]+ecgradd*(t-50))) - 1;
             population:= (1+0.01*(pgrowth^[t-1]+pgadd*(t-50)))*population;
             energint:= (1 - 0.01*(aeei^[t] + aeeiadd*(t-50)) - reei)*energint;
             emissint:= (1 - 0.01*(acei^[t] + aceiadd*(t-50)) - rcei)*emissint;
             watech:= watech*(1-watechrate);
             SO2:= SO2*pow(1+0.01*(pgrowth^[t-1]+pgadd*(t-50)),0.33)
                   *pow(1+0.01*(ypcgrowth^[t-1]+ecgradd*(t-50)),-0.45)
                   *pow(1-0.01*(aeei^[t]+aeeiadd*(t-50))-reei-0.01*(acei^[t]+aceiadd*(t-50))-rcei,1.02);
          END
          ELSE BEGIN
               ygrowth:= (1+0.01*pgrowth^[t-1])*(1+0.01*ypcgrowth^[t-1]) - 1;
               population:= (1+0.01*pgrowth^[t-1])*population;
               energint:= (1 - 0.01*aeei^[t] - reei)*energint;
               emissint:= (1 - 0.01*acei^[t] - rcei)*emissint;
               SO2:= SO2*pow(1+0.01*pgrowth^[t-1],0.33)
                     *pow(1+0.01*ypcgrowth^[t-1],-0.45)
                     *pow(1-0.01*aeei^[t]-0.01*acei^[t],1.02);
          END;
          income:= (1 + ygrowth - taxmp*ryg - ch4cost - n2ocost)*income - formp*forcost*income/GDP0;
          IF (t > 50) AND (t <= 60) THEN
             income:= income + oda;
          popold:= population;
          IF population < 0.000000001 THEN
             YpC:= 0
          ELSE
              YpC:= 1000*income/population;
          agrish:= agrish0*pow(ypc/ypc90,-agel);
          energuse:= (1-seei)*energint*income;
          emission:= (1-scei)*emissint*energuse;
          IF income > 0 THEN BEGIN
             cummemm:= cummemm+emission;
             globcum:= globcum+emission
          END
          ELSE BEGIN
               globcum:= globcum-cummemm;
               cummemm:= 0
          END;
          IF ((t mod 10 = 0) AND (t > 40) AND (t < 149)) OR ((t mod 25 = 0) AND (t > 149)) THEN BEGIN
             decade:= decade + 1;
             htax:= tax[decade];
             IF t=NYear THEN
                htax:= 0;
             taxpar:= (*0.5*)(2.24 - 0.24*root(emission/income-minint))*(0.4+0.6*perm);
             htax:= MC*pow(1.05,t-50)*emission*know*globknow/2/taxpar/income/1000;

(*             htax:= 15*ypc/economy[1].ypc90*emission*know*globknow/2/taxpar/income/1000;*)

(*             IF ypc > 1250 THEN
                htax:= 0.025 + 0.01*(pgrowth^[t] + ypcgrowth^[t] - aeei^[t] - acei^[t]);
             MC:= htax/emission/know/globknow*2*taxpar*income*1000; *)
             ch4red[decade]:= 21*22/6/1000*MC*pow(1.05,t-50)*ch4em^[t]/2/ch4par1/ch4par2/ch4par2/income/1000;
             n2ored[decade]:= 310*7/6/1000*MC*pow(1.05,t-50)*n2oem^[t]/2/n2opar1/n2opar2/n2opar2/income/1000;
             IF htax < 0 THEN
                htax:= 0;
             IF htax > 0.99 THEN
                htax:= 0.99;
             IF ch4red[decade] < 0 THEN
                ch4red[decade]:= 0;
             IF ch4red[decade] > 0.99 THEN
                ch4red[decade]:= 0.99;
             IF n2ored[decade] < 0 THEN
                n2ored[decade]:= 0;
             IF n2ored[decade] > 0.99 THEN
                n2ored[decade]:= 0.99;
             ryg:= taxpar*quad(htax)/know/globknow;
             reei:= perm*0.5*htax;
             rcei:= perm*0.5*htax;
(*             ch4cost:= ch4par1*(expo(ch4par2*ch4red[decade])-1); *)
             ch4cost:= ch4par1*ch4par2*ch4par2*ch4red[decade]*ch4red[decade];
(*             n2ocost:= n2opar1*(expo(n2opar2*n2ored[decade])-1);     *)
             n2ocost:= n2opar1*n2opar2*n2opar2*n2ored[decade]*n2ored[decade];
             IF NOT ((name = 'USA') OR (name = 'CAN') OR (name = 'WEU') OR (name = 'JPK') OR (name = 'ANZ')) THEN BEGIN
                noemer:= 0;
                teller:= 0;
                FOR i:= 1 TO 5 DO BEGIN
                    noemer:= noemer + economy[i].income;
                    teller:= teller + economy[i].ryg*economy[i].income
                END;
                IF noemer > 0.0000001 THEN
                   ryg:= ryg + trade[decade]*teller/noemer
             END
          END;
          seei:= 0.9*seei + (1-perm)*0.5*htax*1.7;
          scei:= 0.9*scei + (1-perm)*0.5*htax*1.7;
          know:= know*root(1+knowpar*htax);
          globknow:= globknow*root(1+knowgpar*htax);
          setinflfor(country,t);

          species:= ((1-bioshare)*spbm+bioshare*biodiv*spbm)/valbase*
                    ypc/valinc/(1+ypc/valinc)/valadj*ypc*population/1000*
                    regdt/bregtmp/dbsta/(1+regdt/bregtmp/dbsta);
          water:= wrbm*watech*GDP90*pow(income/GDP90,wrel)*pow(regtmp/bregtmp,wrnl);
	  IF water > 0.1*income THEN
	     water:= 0.1*income;
          forests:= forbm*income*pow(ypc/ypc90,-forel)*(0.5*pow(regtmp/bregtmp,fornl)+0.5*ln(acco2^[t-1]/co2pre)*forco2);
          IF forests > 0.01*income THEN
             forests:= 0.01*income;
          heating:= hebm*cumaeei*(1-seei)*GDP90*pow(regtmp/bregtmp,henl)*pow(ypc/ypc90,heel)*population/pop90;
          IF heating > 0.05*income THEN
	     heating:= 0.05*income;
          cooling:= cebm*cumaeei*(1-seei)*GDP90*pow(regtmp/bregtmp,cenl)*pow(ypc/ypc90,ceel)*population/pop90;
          IF t > 50 THEN
             cumaeei:= cumaeei*(1-0.01*(aeei^[t]+aeeiadd*(t-50))-reei);
          dengue:= dfbs*population*dfch*pow(regtmp/bregtmp,dfnl)*pow(ypc/ypc90,vbel);
          schisto:= smbs*population*smch*pow(regtmp/bregtmp,smnl)*pow(ypc/ypc90,vbel);
          IF schisto < -smbs*population*pow(ypc/ypc90,vbel) THEN
             schisto:= -smbs*population*pow(ypc/ypc90,vbel);
          malaria:= malbs*population*malch*pow(regtmp/bregtmp,malnl)*pow(ypc/ypc90,vbel);
          IF (temp90-0.49*bregtmp) > 0 THEN BEGIN
             diadead:= diamort*population*pow(ypc/ypc90,diamortel)
                       *(pow((temp90-0.49*bregtmp+regtmp)/(temp90-0.49*bregtmp),diamortnl)-1);
             diasick:= diayld*population*pow(ypc/ypc90,diayldel)
                       *(pow((temp90-0.49*bregtmp+regtmp)/(temp90-0.49*bregtmp),diayldnl)-1)
          END
          ELSE BEGIN
               diadead:= 0;
               diasick:= 0
          END;
          agrate:= agrbm*pow(regdt/bregtmp/dbst,agnl) + (1-1/agtime)*agrate;
          aglevel:= aglparl*regtmp/bregtmp + aglparq*pow(regtmp/bregtmp,2);
          agco2:= agcbm*ln(acco2^[t-1]/co2pre);
          agcost:= (agrate+aglevel+agco2)*agrish*income;
          IF agcost > agrish*income THEN
             agcost:= agrish*income;

          plus:= plus90*pow(ypc/ypc90,plusel);
	  IF plus > 1 THEN
	     plus:= 1;
          urbpop:= (0.031*root(ypc)-0.011*root(popdens))/(1+0.031*root(ypc)-0.011*root(popdens))
                   /(1+urbcorr/(1+0.001*pow(time-40,2)));
          basecardvasc:= cardvasc90 + cvlin*(plus-plus90);
	  IF basecardvasc > 1 THEN
	     basecardvasc:= 1;
          baseresp:= resp90 + rlin*(plus-plus90);
	  IF baseresp > 1 THEN
	     baseresp:= 1;

          cardheat:= (chplbm*plus + chmlbm*(1-plus))*regtmp/bregtmp +
                     (chpqbm*plus + chmqbm*(1-plus))*pow(regtmp/bregtmp,2);
          cardheat:= cardheat*urbpop*population*10;
          IF cardheat > 1000*maxcardvasc*basecardvasc*urbpop*population THEN
             cardheat:= 1000*maxcardvasc*basecardvasc*urbpop*population;
          IF cardheat < 0 THEN
             cardheat:= 0;

          resp:= rlbm*regtmp/bregtmp + rqbm*pow(regtmp/bregtmp,2);
          resp:= resp*urbpop*population*10;
          IF resp > 1000*maxcardvasc*baseresp*urbpop*population THEN
             resp:= 1000*maxcardvasc*baseresp*urbpop*population;
          IF resp < 0 THEN
             resp:= 0;

          cardcold:= (ccplbm*plus + ccmlbm*(1-plus))*regtmp/bregtmp +
                     (ccpqbm*plus + ccmqbm*(1-plus))*pow(regtmp/bregtmp,2);
          cardcold:= cardcold*population*10;
          IF cardcold < -1000*maxcardvasc*basecardvasc*population THEN
             cardcold:= -1000*maxcardvasc*basecardvasc*population;
          IF cardcold > 0 THEN
             cardcold:= 0;

          dead:= dengue + schisto + malaria + cardheat + cardcold + resp + diadead;
(*writeln(dengue:10:2,' ',schisto:10:2,' ',malaria:10:2,' ',cardheat:10:2,' ',cardcold:10:2,' ',resp:10:2,' ',diadead:10:2);*)

          yll:= d2ld*dengue+d2ls*schisto+d2lm*malaria+d2lc*cardheat+d2lc*cardcold+d2lr*resp;
          yld:= d2dd*dengue+d2ds*schisto+d2dm*malaria+d2dc*cardheat+d2dc*cardcold+d2dr*resp+diasick;
          deadcost:= vsl*ypc*dead/1000000000;
          (*deadcost:= vyll*ypc*yll/1000000000;*)
          morbcost:= vmorb*ypc*yld/1000000000;

          wetval:= wvbm/valbase*ypc/valinc/(1+ypc/valinc)/valadj*ypc;
          IF (wetmax > 0) THEN
             IF wettot > wetmax*(1-bioshare/(1+bioshare)) THEN
                wetval:= wetval*2
             ELSE
                 wetval:= wetval*((1-bioshare)+bioshare*wetmax/(wetmax-wettot));
          dryval:= dvbm*popdens*ypc/1000/incdens;
          npprotcost:= pc*ds*100*(1+discrate+ygrowth)/(discrate+ygrowth);
          IF npprotcost < 0 THEN
             npprotcost:= 0;
          npwetcost:= wetval*wmbm*ds/(discrate+ygrowth);
          IF npwetcost < 0 THEN
             npwetcost:= 0;
          npdrycost:= dryval*dlbm*ds*(1+ygrowth)*(discrate+ygrowth)/discrate/discrate;
          IF npdrycost < 0.00000001 THEN BEGIN
             npdrycost:= 0;
             protlev:= 0
          END
          ELSE
              protlev:= 1-0.5*(npprotcost+npwetcost)/npdrycost;
          IF protlev < 0 THEN
             protlev:= 0;
          wetland:= wlbm*sea;
          IF ds > 0 THEN
             wetmeas:= wmbm*ds*protlev + wetmeas;
          wettot:= wetland + wetmeas;
          IF ds > 0 THEN
             wetann:= (wlbm + wmbm*protlev)*ds
          ELSE
              wetann:= 0;
          IF wettot > wetmax THEN BEGIN
             wettot:= wetmax;
             wetann:= 0
          END;
          wetcost:= wetval*wetann/1000 + wetdel*wetcost;
          IF ds > 0 THEN BEGIN
             dryland:= dlbm*ds*(1-protlev) + dryland;
             area:= area - dlbm*ds*(1-protlev);
             drycost:= dryval*dlbm*ds*(1-protlev)/1000 + drydel*drycost;
          END;
	  IF drycost < 0 THEN
	     drycost:= 0;
          protcost:= protlev*pc*ds/10;
	  IF protcost < 0 THEN
	     protcost:= 0;
          enter:= 0;
          FOR j:= 1 TO NoReg DO
              enter:= enter + immigrate[j]*economy[j].leave;
          leave:= coastpd*popdens*dlbm*ds*(1-protlev);
          IF leave < 0 THEN
             leave:= 0;
          cumleave:= leave + cumleave;
          cumenter:= enter + cumenter;
          leavecost:= emcst*ypc*leave/1000000000 + leavedel*leavecost;
          entercost:= immcst*ypc*enter/1000000000 + (1-1/entertime)*entercost;

          eloss:= 1*(- water - forests - heating - cooling - agcost + drycost + protcost + entercost);
          IF eloss > income THEN
             eloss:= income;
          sloss:= 1*(species + deadcost + morbcost + wetcost + leavecost);
(*writeln(species:10:2,' ',deadcost:10:2,' ',morbcost:10:2,' ',wetcost:10:2,' ',leavecost:10:2);*)
(*writeln(name,' ',t+1950,' ',eloss:10:2,' ',sloss:10:2,' ',ypc:10:2,' ',population:10:2,' ',income:10:2); *)
(*readln;*)
          IF t >= 40 THEN
             population:= population + (enter - leave)/1000000;
          IF dead > 0 THEN
             population:= population - dead/1000000;
	  IF population <= 0.000001 THEN
	     population:= 0.000001;
          popdens:= population/area*1000000;
          welfare:= (1-consleak)*income - eloss;
(*          IF t >= 40 THEN
             income:= income - consleak*eloss/10; *)
          IF (population < 0.00000001) OR ((sloss+kant) >= welfare) OR (welfare <= 0) THEN
             welfare:= -100000000
          ELSE BEGIN
              welfare:= 1000*(welfare-sloss-kant)/population;
              IF riskave = 1 THEN
                 welfare:= log(welfare)
              ELSE
                  welfare:= pow(welfare,1-riskave)/(1-riskave)
          END;
          IF income < 0.1*population THEN
             income:= 0.1*population;
          eloss:= eloss/income;
          sloss:= sloss/income;
          IF ((optper < 11) AND (t = 40+10*optper)) OR ((optper > 10) AND (t = 25*optper-125)) THEN BEGIN
             cdr:= 1;
             hNPVW:= welfare
          END;
          IF ((optper < 11) AND (t > 40+10*optper)) OR ((optper > 10) AND (t > 25*optper-125))THEN BEGIN
             cdr:= (1+discrate)*cdr;
             IF hNPVW >= 1.0E-309 THEN
                hNPVW:= hNPVW + welfare/cdr
          END
     END
END;

PROCEDURE Write1(VAR country: regrec; t: word; VAR els,sls,
                     ent,mal,sch,den,car,res,lea,inc,wel,pop,mc: text);

BEGIN
     WITH country, climate DO BEGIN
          write(ent,enter:10:4,' ');
          write(lea,leave:10:4,' ');
          write(mal,malaria:10:4,' ');
          write(sch,schisto:10:4,' ');
          write(den,diadead:10:4,' ');
          write(car,cardheat+cardcold:10:4,' ');
          write(res,resp:10:4,' ');
          IF income > 0.000001 THEN BEGIN
             write(els,eloss*100:8:4,' ');
             write(sls,sloss*100:8:4,' ');
             IF t >= 40 THEN BEGIN
                write(mc,2*taxpar*tax[trunc((t-40)/10)]*income*1000/emission/know/globknow:8:2,' ');
write(mc,-taxpar*tax[trunc((t-40)/10)]*income*1000/know/globknow/emission/root(1+tax[trunc((t-40)/10)]):8:2,' ')
             END
             ELSE
                 write(mc,0,' ',0,' ');
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ');
               write(mc,0,' ')
          END;
          write(inc,income:6:0,' ');
          write(wel,emission:6:0,' ');
          write(pop,population:6:0,' ');
     END
END;

PROCEDURE WriteM(VAR country: regrec; t: word; VAR els,sls,inc,pop: text);

BEGIN
     WITH country, climate DO BEGIN
          IF income > 0.000001 THEN BEGIN
             write(els,eloss*100:14:10,' ');
             write(sls,sloss*100:14:10,' ');
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ');
          END;
          write(inc,income:12:6,' ');
          write(pop,population:12:6,' ');
     END
END;

PROCEDURE CalcClimate(j: word);
(* calculates the climatic variables at time J *)

VAR mco2, emfforest: double;
    i, dec: word;
    globch4, globn2o: double;

BEGIN
     dec:= economy[1].decade;
     WITH climate DO BEGIN
          mco2:= 0;
          FOR i:= 1 TO NoReg DO
              WITH economy[i] DO BEGIN
                   mco2:= mco2 + emission + forestemm^[j]
              END;
          IF j > 50 THEN BEGIN
             biodiv:= nospecbase/nospecies;
             FOR i:= 1 TO NoReg DO
                 WITH economy[i] DO
                      mco2:= mco2 + foremadd*(j-50) - forestry[dec]
          END;
          FOR i:= 1 TO 5 DO
              cbox[i]:= cbox[i]*decay[i] + 0.000471*frac[i]*(mco2 + addmp*addco2^[j]);
          acco2^[j]:= cbox[1];
          FOR i:= 2 TO 5 DO
              acco2^[j]:= acco2^[j] + cbox[i];
          globch4:= 0;
          globn2o:= 0;
          globso2:= 34.4;
          FOR i:= 1 TO NoReg DO BEGIN
              IF j > 50 THEN BEGIN
                 globch4:= globch4 + economy[i].ch4em^[j]*(1-economy[i].ch4red[dec]);
                 globn2o:= globn2o + economy[i].n2oem^[j]*(1-economy[i].n2ored[dec]);
                 globso2:= globso2 + economy[i].so2*(1-economy[i].seei-economy[i].scei)
              END
              ELSE BEGIN
                   globch4:= globch4 + economy[i].ch4em^[j];
                   globn2o:= globn2o + economy[i].n2oem^[j];
                   globso2:= globso2 + economy[i].so2
              END
          END;
          IF j > 50 THEN BEGIN
             acch4^[j]:= acch4^[j-1] + 0.3597*(globch4 + ch4add*(j-50) + addmp*addch4^[j])
                         - ch4decay*(acch4^[j-1]-ch4pre);
             acn2o^[j]:= acn2o^[j-1] + 0.2079*(globn2o + n2oadd*(j-50) + addmp*addn2o^[j])
                         - n2odecay*(acn2o^[j-1]-n2opre)
          END
          ELSE BEGIN
               acch4^[j]:= acch4^[j-1] + 0.3597*(globch4 + addch4^[j])
                           - ch4decay*(acch4^[j-1]-ch4pre);
               acn2o^[j]:= acn2o^[j-1] + 0.2079*(globn2o + addn2o^[j])
                         - n2odecay*(acn2o^[j-1]-n2opre)
          END;
          IF acch4^[j] < 0 THEN
             acch4^[j]:= 0;
          IF acn2o^[j] < 0 THEN
             acn2o^[j]:= 0;
          rfCO2:= 6.3*log(acco2^[j]/co2pre);
          rfCH4:= 0.036*(sqrt(acCH4^[j])-sqrt(ch4pre))
                       - interact(acch4^[j],n2opre) + CH4N2O;
          rfN2O:= 0.14*(sqrt(acN2O^[j])-sqrt(N2Opre))
                       - interact(ch4pre,acn2o^[j]) + CH4N2O;
          rfSO2:= 0.3*globSO2/14.6 + 0.8*ln(1+globSO2/34.4)/ln(1+14.6/34.4) - 3.0;
          radforc:= rfCO2 + rfCH4 + rfN2O - rfSO2;
          IF radforc+rfSO2 > maxRF THEN BEGIN
             maxRF:= radforc+rfSO2;
             maxYR:= j+1950
          END;
          dt:= delaytemp*temps*radforc - delaytemp*temp;
          temp:= dt + temp;
          FOR i:= 1 TO NoReg DO BEGIN
              IF scengmt THEN BEGIN
                 economy[i].regtmp:= temp*economy[i].bregtmp + economy[i].scentemp^[j];
                 economy[i].regdt:= dt*economy[i].bregtmp + economy[i].scentemp^[j] - economy[i].scentemp^[j-1];
                 IF economy[i].regdt < 0 THEN
                    economy[i].regdt:= -economy[i].regdt
              END
              ELSE BEGIN
                   economy[i].regtmp:= temp*economy[i].bregtmp;
                   economy[i].regdt:= dt*economy[i].bregtmp;
                   IF economy[i].regdt < 0 THEN
                      economy[i].regdt:= -economy[i].regdt
              END
          END;
          IF j > 50 THEN
             nospecies:= nospecies*(1-bioloss-biosens*dt*dt/dbsta/dbsta);
(*          IF j < 150 THEN *)
             ds:= delaysea*seas*temp - delaysea*sea
(*          ELSE
              ds:= delaysea*(seas*temp + (j-150)*0.025) - delaysea*sea + 0.025*);
          sea:= ds + sea;
          IF j = 40+10*optper THEN
             toptper:= temp
     END
END;

PROCEDURE DrawRandom;

VAR i, j: word;
    hulp, fake,immsum: double;

(*bioloss, bioshare, biosense, watech, agriculture*)

BEGIN
    WITH climate DO BEGIN
         GenNormal(0,1,ch4add,n2oadd);
         ch4add:= ch4add*ch4sd;
         n2oadd:= n2oadd*n2osd;

(*         GenGamma(8.1270,0.3508,temps); (*mode 2.5; var 1*)
GenGamma(2.52,3,temps);
(*         GenGamma(4.9218,0.6375,temps); (*mode 2.5; var 2*)
(*         GenGamma(4.27,1.16,temps);       (*mode 3.8; sd 2.4*)
         temps:= temps/6.3/ln(2);
(*         GenGamma(6.3584,0.0317,seas); (* mode 0.17; var 0.08^2 *)
(*         GenGamma(4.0083,0.0565,seas); (* mode 0.17; var 2*0.08^2 *)
(*         GenGamma(7.4729,0.0263,seas);      (* mode 0.17; sd 0.07 *)
         GenGamma(5.9959077,0.0612581,seas); (* mode 0.30604; var 0.15^2 *)
         GenTriang(8,16,1/ch4dec,hulp);
         ch4decay:= 1/hulp;
         GenTriang(100,170,1/n2odec,hulp);
         n2odecay:= 1/hulp;
         GenTriang(0.5*lifetemp,2*lifetemp,lifetemp,hulp);
         delaytemp:= 1/hulp;
         GenTriang(0.5*lifesea,2*lifesea,lifesea,hulp);
         delaysea:= 1/hulp;
         GenNormGTZ(lifeco2,0.25*lifeco2,hulp);
         decay[2]:= exp(-1/hulp);
         GenNormGTZ(lifeco3,0.25*lifeco3,hulp);
         decay[3]:= exp(-1/hulp);
         GenNormGTZ(lifeco4,0.25*lifeco4,hulp);
         decay[4]:= exp(-1/hulp);
         GenNormGTZ(lifeco5,0.25*lifeco5,hulp);
         decay[5]:= exp(-1/hulp);
    	 GenNormGTZ(biolossbg,biolosssd,bioloss);
    	 GenNormGTZ(biosensbg,biosenssd,biosens);
    	 GenTriang(0,1,biosharebg, bioshare);
    END;
    GenNormGTZ(watechratebg,watechratesd,watechrate);

    FOR i:= 1 TO NoReg DO
        WITH economy[i] DO BEGIN
             gennormal(0,1,pgadd,ecgradd);
             pgadd:= pgadd*pgsd;
             ecgradd:= ecgradd*ecgrsd;
             gennormal(0,1,aeeiadd,aceiadd);
             aeeiadd:= aeeiadd*aeeisd;
             aceiadd:= aceiadd*aceisd;

             gennormal(0,1,foremadd,fake);
             foremadd:= foremsd*foremadd;

             gennormGTZ(spbmbg,spbmsd,spbm);
             gennormGTZ(vslbg,vslsd,vsl);

             GenNormGTZ(vmorbbg,vmorbsd,vmorb);
             gennormGTZ(vyllbg,vyllsd,vyll);

             gennormGTZ(valincbg,valincsd,valinc);
             valadj:= valbase/valinc/(1+valbase/valinc);

             gennormGTZ(malchbg,malchsd,malch);
             gennormGTZ(malnlbg,malnlsd,malnl);
             gennormGTZ(dfchbg,dfchsd,dfch);
             gennormGTZ(dfnlbg,dfnlsd,dfnl);
             gennormGTZ(-smchbg,smchsd,smch);
             smch:= -smch;
             gennormGTZ(smnlbg,smnlsd,smnl);
             gennormGTZ(-vbelbg,vbelsd,vbel);
             vbel:= -vbel;
             gennormGTZ(forco2bg,forco2sd,forco2);

             GenNormGTZ(-diamortelbg,diamortelsd,diamortel);
             diamortel:= -diamortel;
             gennormGTZ(diamortnlbg,diamortnlsd,diamortnl);
             GenNormGTZ(-diayldelbg,diayldelsd,diayldel);
             diayldel:= -diayldel;
             gennormGTZ(diayldnlbg,diayldnlsd,diayldnl);

             gennormal(0,1,chplbm,chmlbm);
             gennormal(0,1,chpqbm,chmqbm);
             gennormal(0,1,ccplbm,ccmlbm);
             gennormal(0,1,ccpqbm,ccmqbm);
             gennormal(0,1,rlbm,rqbm);
             chplbm:= ccplbmbg + chplbmsd*chplbm;
             chmlbm:= chmlbmbg + chmlbmsd*chmlbm;
             chpqbm:= chpqbmbg + chpqbmsd*chpqbm;
             chmqbm:= chmqbmbg + chmqbmsd*chmqbm;
             ccplbm:= ccplbmbg + ccplbmsd*ccplbm;
             ccmlbm:= ccmlbmbg + ccmlbmsd*ccmlbm;
             ccpqbm:= ccpqbmbg + ccpqbmsd*ccpqbm;
             ccmqbm:= ccmqbmbg + ccmqbmsd*ccpqbm;
             rlbm:= rlbmbg + rlbmsd*rlbm;
             rqbm:= rqbmbg + rqbmsd*rqbm;
             gennormGTZ(cvlinbg,cvlinsd,cvlin);
             gennormGTZ(rlinbg,rlinsd,rlin);
             gennormGTZ(pluselbg,pluselsd,plusel);
             gennormGTZ(maxcardvascbg,maxcardvascsd,maxcardvasc);

             gennormal(0,1,wrbm,forbm);
             gennormGTZ(wrelbg,wrelsd,wrel);
             gennormGTZ(wrnlbg,wrnlsd,wrnl);
             gennormGTZ(forelbg,forelsd,forel);
             gennormGTZ(fornlbg,fornlsd,fornl);
             gennormGTZ(hebmbg,hebmsd,hebm);
	     gennormGTZ(-cebmbg,cebmsd,cebm);
             cebm:= -cebm;
             gennormGTZ(heelbg,heelsd,heel);
             gennormGTZ(henlbg,henlsd,henl);
             gennormGTZ(ceelbg,ceelsd,ceel);
             gennormGTZ(cenlbg,cenlsd,cenl);
             wrbm:= wrbmbg + wrbmsd*wrbm;
             forbm:= forbmbg + forbmsd*forbm;

             gennormGTZ(-agrbmbg,agrbmsd,agrbm);
             agrbm:= -agrbm;
             gennormGTZ(agtimebg-1,agtimesd,agtime);
             agtime:= agtime+1;
             gennormGTZ(agnlbg,agnlsd,agnl);
             gennormGTZ(agelbg,agelsd,agel);
             gennormal(0,1,aglbm,agopt);
             gennormGTZ(agcbmbg/ln(2),agcbmsd/ln(2),agcbm);
             aglbm:= aglbmbg + aglbmsd*aglbm;
             agopt:= agoptbg + agoptsd*agopt;
             aglparl:= -2*agopt*aglbm/(10.24-6.4*agopt);
             aglparq:= aglbm/(10.24-6.4*agopt);
             IF aglparq > 0 THEN BEGIN
                aglparq:= 0;
                aglparl:= aglbm/3.2
             END;

             gennormGTZ(wvbmbg,wvbmsd,wvbm);
             gennormGTZ(dvbmbg,dvbmsd,dvbm);
             gennormGTZ(wlbmbg,wlbmsd,wlbm);
             gennormGTZ(wmbmbg,wmbmsd,wmbm);
             gennormGTZ(pcbg,pcsd,pc);
             gennormGTZ(dlbmbg,dlbmsd,dlbm);

             genexponential(wetdelsd,wetdel);
             WHILE wetdel > 1 DO
                   genexponential(wetdelsd,wetdel);
             genexponential(drydelsd,drydel);
             WHILE drydel > 1 DO
                   genexponential(drydelsd,drydel);
             genexponential(leavedelsd,leavedel);
             WHILE leavedel > 1 DO
                   genexponential(leavedelsd,leavedel);

             FOR j:= 1 TO NoReg DO
                 genexponential(immigratesd[j],immigrate[j]);
             gennormGTZ(immigratebg[i],immigratesd[i],immigrate[i]);
             gennormGTZ(entertimebg-1,entertimesd,entertime);
             entertime:= entertime+1;
             immsum:= 0;
             FOR j:= 1 TO NoReg DO
                 immsum:= immsum + immigrate[j];
             FOR j:= 1 TO NoReg DO
                 immigrate[j]:= immigrate[j]/immsum;

             gennormGTZ(emcstbg,emcstsd,emcst);
             gennormGTZ(immcstbg,immcstsd,immcst);

             gengamma(5.8284,0.2071,taxmp);
             gengamma(5.8284,0.2071,formp);
(*             gengamma(5.8284,0.2071,cregmp);  (* var = 0.25, mode = 1 *)
(*             gengamma(9.8990,0.1124,taxmp);
             gengamma(9.8990,0.1124,formp);
             gengamma(9.8990,0.1124,cregmp);   (* var = 0.125, mode = 1 *)
        END;
    hulp:= 4.62;
    GenExponential(economy[7].trade[1], hulp);
    hulp:= 2.63;
    GenExponential(economy[8].trade[1], hulp);
    hulp:= 0.53;
    GenExponential(economy[9].trade[1], hulp);
    GenExponential(economy[10].trade[1], hulp);
    GenExponential(economy[13].trade[1], hulp);
    GenExponential(economy[16].trade[1], hulp);
    hulp:= 0.46;
    GenExponential(economy[11].trade[1], hulp);
    GenExponential(economy[12].trade[1], hulp);
    hulp:= 1.08;
    GenExponential(economy[14].trade[1], hulp);
    GenExponential(economy[15].trade[1], hulp);
    FOR i:= 7 TO NoReg DO BEGIN
        economy[i].trade[2]:= economy[i].trade[1]*4/5;
        economy[i].trade[3]:= economy[i].trade[1]*3/5;
        economy[i].trade[4]:= economy[i].trade[1]*2/5;
        economy[i].trade[5]:= economy[i].trade[1]*1/5
    END
END;

PROCEDURE RunEconomy;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, run : word;
    worldemm, oecdemm, ann1emm, maxkant, loss, totloss: double;

BEGIN
(*write('.');*)
     IF uncertain THEN BEGIN
        randseed:= 34514325;
        FOR run:= 1 TO runs DO BEGIN
(*gotoxy(3,23);*)write(run:5);
            DrawRandom;
            ReInit;
            FOR i:= 1 TO NYear DO BEGIN
                IF i < 40 THEN
                   minint:= 1000;
                FOR j:= 1 TO NoReg DO BEGIN
                    Calculate(economy[j],i);
                    IF economy[j].emission/economy[j].income < minint THEN
                       minint:= economy[j].emission/economy[j].income
                END;
                IF ((i < 150) AND (i = 49+10*optper)) OR (i=149+25*optper) THEN
                   FOR j:= 1 TO NoReg DO BEGIN
                       emmrec[j]:= emmrec[j]*(run-1)/run + economy[j].emission/run;
                       IF economy[j].emission > ncemmres[j,optper] THEN
                          count[j]:= count[j]+1
                   END;
                CalcClimate(i)
            END;
            FOR i:= 1 TO NoReg DO
                economy[i].NPVW:= economy[i].hNPVW/run +
                                     (run-1)*economy[i].NPVW/run
        END
     END
     ELSE BEGIN
          InitImpactPar;
          ReInit;
          FOR i:= 1 TO NYear DO BEGIN
              maxkant:= 0;
              totloss:= 0;
              IF i < 40 THEN
                 minint:= 1000;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  totloss:= totloss + loss*economy[j].income;
                  IF loss > maxkant THEN
                     maxkant:= loss;
                  IF economy[j].emission/economy[j].income < minint THEN
                     minint:= economy[j].emission/economy[j].income
              END;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                       END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss) - totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              IF ((i < 150) AND (i = 49+10*optper)) OR (i=149+25*optper) THEN BEGIN
                 worldemm:= 0;
                 oecdemm:= 0;
                 FOR j:= 1 TO NoReg DO BEGIN
                     IF j < 4 THEN
                        oecdemm:= oecdemm + economy[j].emission;
                     worldemm:= worldemm + economy[j].emission (*+
                                economy[j].foremp*economy[j].forestemm^[i] -
                                economy[j].forestry[trunc(i/10)]*);
                     emmrec[j]:= economy[j].emission -
                                 gwpch4*economy[j].ch4em^[i]*economy[j].ch4red[trunc(i/10)] -
                                 gwpn2o*economy[j].n2oem^[i]*economy[j].n2ored[trunc(i/10)]
                 END;
                 ann1emm:= oecdemm + economy[4].emission (*- economy[2].emission*);
                 worldemm:= worldemm (*- economy[5].emission - economy[6].emission - economy[9].emission*);
                 globviol:= (worldemm > cemmres[optper])
              END;
              CalcClimate(i)
          END;
          C2200:= climate.acco2^[NYear];
          FOR i:= 1 TO NoReg DO
(*              IF C2200 > CMax THEN
                 economy[i].NPVW:= economy[i].hNPVW*
                                   (1-0.5*(noper-optper)/noper*(C2200-CMax)*(C2200-CMax)/(CBaU-CMax)/(CBaU-CMax))
              ELSE *)
                  economy[i].NPVW:= economy[i].hNPVW
     END
END;
(*
PROCEDURE WriteEMF;

VAR i, j, k, run : word;
    oecd,noecd,clim:text;
    TCEm, TCFE, TInc, TCCC, TERC: double;

BEGIN
     InitImpactPar;
     ReInit;
     assign(oecd,sessionpathname+'oecd.out');
     rewrite(oecd);
     assign(noecd,sessionpathname+'noecd.out');
     rewrite(noecd);
     assign(clim,sessionpathname+'climate.out');
     rewrite(clim);
     FOR i:= 1 TO NYear DO BEGIN
         IF i < 40 THEN
            minint:= 1000;
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             IF economy[j].emission/economy[j].income < minint THEN
                minint:= economy[j].emission/economy[j].income
         END;
         CalcClimate(i);
         IF (*(i mod 10 = 0)*) (*0=0 THEN BEGIN
            TCEm:= 0;
            TCFE:= 0;
            TInc:= 0;
            TCCC:= 0;
            TERC:= 0;
            FOR j:= 1 TO 5 DO
                WITH economy[j] DO BEGIN
                     TCEm:= TCEm + emission;
                     TCFE:= TCFE + forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income;
                     TCCC:= TCCC + (eloss+sloss)*income;
                     TERC:= TERC + 0.01*ryg*income + formp*forcost*income/GDP0
                END;
            writeln(oecd,TCEm:12:6,' ',TCFE:12:6,' ',TInc:12:6,' ',TCCC:12:6,' ',TERC:12:6);
            TCEm:= 0;
            TCFE:= 0;
            TInc:= 0;
            TCCC:= 0;
            TERC:= 0;
            FOR j:= 6 TO NoReg DO
                WITH economy[j] DO BEGIN
                     TCEm:= TCEm + emission;
                     TCFE:= TCFE + forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income;
                     TCCC:= TCCC + (eloss+sloss)*income;
                     TERC:= TERC + 0.01*ryg*income + formp*forcost*income/GDP0
                END;
            writeln(noecd,TCEm:12:6,' ',TCFE:12:6,' ',TInc:12:6,' ',TCCC:12:6,' ',TERC:12:6);
            WITH Climate DO BEGIN
                 write(clim,i+1950:4,' ',acco2^[i]:10:6,' ',acch4^[i]:10:6,' ',acn2o^[i]:10:6,' ',temp:10:6,' ');
                 writeln(clim,dt:6:4,' ',sea:12:6)
            END
         END;
     END;
     close(oecd);
     close(noecd);
     close(clim)
END;
*)(*
PROCEDURE WriteTol;

VAR i, j, k, run : word;
    econ,clim:text;
    TCEm, TCFE, TInc, TCCC, TERC: double;

BEGIN
     InitImpactPar;
     ReInit;
     assign(econ,sessionpathname+'tolecon.out');
     rewrite(econ);
     assign(clim,sessionpathname+'tolclim.out');
     rewrite(clim);
     FOR j:= 1 TO NoReg DO
         WITH economy[j] DO BEGIN
              TCFE:= forestemm^[0];
              TERC:= 0.01*ryg*income + forcost;
              IF (income > 0.0000001) AND (population > 0.0000001) THEN BEGIN
               write(econ,1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
                 write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
                 writeln(econ,TERC:12:6,' ',terc/income*100:12:6,' ',name)
              END
              ELSE BEGIN
               write(econ,1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
                   write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
                   writeln(econ,TERC:12:6,' ',0,' ',name)
              END
         END;
     WITH Climate DO BEGIN
          writeln(clim,1950:4,' ',acco2^[20]:12:6,' ',temp:12:6,' ',sea:8:6);
     END;
     FOR i:= 1 TO NYear DO BEGIN
         IF i < 40 THEN
            minint:= 1000;
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             IF economy[j].emission/economy[j].income < minint THEN
                minint:= economy[j].emission/economy[j].income
         END;
         CalcClimate(i);
         IF (i=40) OR (i=50) OR (i=75) OR (i=100) OR (i=150) OR (i=250) THEN BEGIN
            FOR j:= 1 TO NoReg DO
                WITH economy[j] DO BEGIN
                     TCFE:= forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income;
                     TERC:= 0.01*ryg*income + formp*forcost*income/GDP0;
IF income > 0.00000001 THEN BEGIN
write(econ,i+1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
writeln(econ,TERC:12:6,' ',terc/income*100:12:6,' ',name)
END
ELSE BEGIN
write(econ,i+1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
writeln(econ,TERC:12:6,' ',0,' ',name)
END
                END;
            WITH Climate DO
                 writeln(clim,i+1950:4,' ',acco2^[i]:12:6,' ',temp:12:6,' ',sea:12:6);
         END;
     END;
     close(econ);
     close(clim)
END;
*)
PROCEDURE RunEconomy1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,eap,sap,ent,mal,sch,den,car,res,lea,inc,wel,pop,clim,mc,dam:text;
    wunc,cunc,tunc,punc,gunc,iunc,eunc,runc: text;
    totpop,totinc,totimp,totemm,totred: double;
    totdam, toty, loss, totloss, maxkant: double;

BEGIN
     writeln('enter runeconomy1');
     IF uncertain THEN BEGIN
        randseed:= 34514325;
        assign(wunc,sessionpathname+'welfunc.out');
        rewrite(wunc);
        assign(cunc,sessionpathname+'concunc.out');
        rewrite(cunc);
        assign(tunc,sessionpathname+'tempunc.out');
        rewrite(tunc);
        assign(punc,sessionpathname+'popunc.out');
        rewrite(punc);
        assign(gunc,sessionpathname+'incunc.out');
        rewrite(gunc);
        assign(iunc,sessionpathname+'impunc.out');
        rewrite(iunc);
        assign(eunc,sessionpathname+'emmunc.out');
        rewrite(eunc);
        assign(runc,sessionpathname+'redunc.out');
        rewrite(runc);
        FOR run:= 1 TO runs DO BEGIN
writeln(run:5);
            DrawRandom;
            ReInit;
            FOR i:= 1 TO NYear DO BEGIN
                IF i < 40 THEN
                   minint:= 1000;
                FOR j:= 1 TO NoReg DO BEGIN
                    Calculate(economy[j],i);
                    IF economy[j].emission/economy[j].income < minint THEN
                       minint:= economy[j].emission/economy[j].income
                END;
                CalcClimate(i);
                IF (i>=50) AND (i mod 25 = 0) THEN BEGIN
                   write(tunc,climate.temp:8:4,' ');
                   totpop:= 0;
                   totinc:= 0;
                   totimp:= 0;
                   totemm:= 0;
                   totred:= 0;
                   FOR k:= 1 TO NoReg DO BEGIN
                       totpop:= totpop + economy[k].population;
                       totinc:= totinc + economy[k].income;
                       totemm:= totemm + economy[k].emission;
                       totimp:= totimp + (economy[k].eloss+economy[k].sloss)*economy[k].income;
                       totred:= totred + economy[k].taxmp*economy[k].ryg*economy[k].income;
                   END;
                   write(punc,totpop:10:4,' ');
                   write(gunc,totinc:10:4,' ');
                   write(eunc,totemm:10:4,' ');
(*                   write(iunc,totimp/totinc*100:8:4,' ');*)
                     write(iunc,economy[15].eloss+economy[15].sloss:8:4,' ');
                   write(runc,totred/totinc*100:10:8,' ')
                END
            END;
            writeln(tunc);
            writeln(punc);
            writeln(gunc);
            writeln(eunc);
            writeln(iunc);
            writeln(runc);
            FOR i:= 1 TO NoReg DO
                WITH Economy[i] DO BEGIN
                     NPVW:= hNPVW/run + (run-1)*NPVW/run;
                     write(wunc,hNPVW:20:4,' ')
                END;
            writeln(wunc);
            WITH climate DO BEGIN
                 c2300:= acco2^[NYear]/run + (run-1)*c2300/run;
                 c2275:= acco2^[NYear-25]/run + (run-1)*c2275/run;
                 c2250:= acco2^[NYear-50]/run + (run-1)*c2250/run;
                 c2225:= acco2^[NYear-75]/run + (run-1)*c2225/run;
                 c2200:= acco2^[NYear-100]/run + (run-1)*c2200/run;
                 c2175:= acco2^[NYear-125]/run + (run-1)*c2175/run;
                 c2150:= acco2^[NYear-150]/run + (run-1)*c2150/run;
                 c2125:= acco2^[NYear-175]/run + (run-1)*c2125/run;
                 c2100:= acco2^[NYear-200]/run + (run-1)*c2100/run;
                 c2075:= acco2^[NYear-225]/run + (run-1)*c2075/run;
                 c2050:= acco2^[NYear-250]/run + (run-1)*c2050/run;
                 c2025:= acco2^[NYear-275]/run + (run-1)*c2025/run;
                 c2000:= acco2^[NYear-300]/run + (run-1)*c2000/run;
                 write(cunc,acco2^[NYear-300]:10:4,' ',acco2^[NYear-275]:10:4,' ');
                 write(cunc,acco2^[NYear-250]:10:4,' ',acco2^[NYear-225]:10:4,' ');
                 write(cunc,acco2^[NYear-200]:10:4,' ',acco2^[NYear-175]:10:4,' ');
                 write(cunc,acco2^[NYear-150]:10:4,' ',acco2^[NYear-125]:10:4,' ');
                 write(cunc,acco2^[NYear-100]:10:4,' ',acco2^[NYear-75]:10:4,' ');
                 write(cunc,acco2^[NYear-50]:10:4,' ',acco2^[NYear-25]:10:4,' ');
                 writeln(cunc,acco2^[NYear]:10:4)
            END
        END;
        close(wunc);
        close(cunc);
        close(tunc);
        close(punc);
        close(eunc);
        close(gunc);
        close(iunc);
        close(runc);
     END
     ELSE BEGIN
          InitImpactPar;
          ReInit;
          assign(els,sessionpathname+'eloss.out');
          rewrite(els);
          assign(sls,sessionpathname+'sloss.out');
          rewrite(sls);
          assign(ent,sessionpathname+'enter.out');
          rewrite(ent);
          assign(mal,sessionpathname+'malaria.out');
          rewrite(mal);
          assign(sch,sessionpathname+'schisto.out');
          rewrite(sch);
          assign(den,sessionpathname+'dengue.out');
          rewrite(den);
          assign(car,sessionpathname+'cardio.out');
          rewrite(car);
          assign(res,sessionpathname+'respir.out');
          rewrite(res);
          assign(lea,sessionpathname+'leave.out');
          rewrite(lea);
          assign(inc,sessionpathname+'income.out');
          rewrite(inc);
          assign(wel,sessionpathname+'emissions.out');
          rewrite(wel);
          assign(pop,sessionpathname+'population.out');
          rewrite(pop);
          assign(clim,sessionpathname+'climate.out');
          rewrite(clim);
          assign(mc,sessionpathname+'margcost.out');
          rewrite(mc);
          assign(dam,sessionpathname+'damage.out');
          rewrite(dam);
          FOR i:= 1 TO NYear DO BEGIN
              write(els,i+1950,' ');
              write(sls,i+1950,' ');
              write(ent,i+1950,' ');
              write(mal,i+1950,' ');
              write(sch,i+1950,' ');
              write(den,i+1950,' ');
              write(car,i+1950,' ');
              write(res,i+1950,' ');
              write(lea,i+1950,' ');
              write(inc,i+1950,' ');
              write(wel,i+1950,' ');
              write(pop,i+1950,' ');
              write(mc,i+1950,' ');
              write(dam,i+1950,' ');
              totdam:= 0;
              toty:= 0;
              maxkant:= 0;
              IF i < 40 THEN
                 minint:= 1000;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  IF loss > maxkant THEN
                     maxkant:= loss;
                  IF economy[j].emission/economy[j].income < minint THEN
                     minint:= economy[j].emission/economy[j].income;
                  Write1(economy[j],i,els,sls,ent,mal,sch,den,car,res,lea,inc,wel,pop,mc);
                  totdam:= totdam + (economy[j].eloss+economy[j].sloss)*economy[j].income;
                  toty:= toty + economy[j].income
              END;
              totloss:= totdam;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                     END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss)-totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              CalcClimate(i);
              WITH climate DO BEGIN
                   write(clim,i+1950:4,' ',acco2^[i]:9:4,' ',acch4^[i]:9:4,' ',acn2o^[i]:9:4,' ',globso2:8:4,' ',radforc:8:4);
                   writeln(clim,' ',temp:11:8,' ',dt:10:8,' ',sea:6:3)
              END;
              writeln(els);
              writeln(sls);
              writeln(ent);
              writeln(mal);
              writeln(sch);
              writeln(den);
              writeln(res);
              writeln(car);
              writeln(lea);
              writeln(inc);
              writeln(wel);
              writeln(pop);
              writeln(mc);
              writeln(dam,climate.temp:8:6,' ',totdam:12:4,' ',toty:12:4,' ',totdam/toty:10:6);
          END;
          close(els);
          close(sls);
          close(ent);
          close(mal);
          close(sch);
          close(den);
          close(car);
          close(res);
          close(lea);
          close(inc);
          close(wel);
          close(pop);
          close(mc);
          close(clim);
          close(dam);
          (*WriteTol;*)
          FOR i:= 1 TO NoReg DO
              economy[i].NPVW:= economy[i].hNPVW
     END
;writeln('exit runeconomy1');
END;

PROCEDURE MargRun1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run: word;
    els,sls,inc,pop,clim: text;
    totpop,totinc,totimp,totemm,totred: double;
    hp,hy,hw: regarr;

BEGIN
     assign(els,sessionpathname+'eloss.txt');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.txt');
     rewrite(sls);
     assign(inc,sessionpathname+'income.txt');
     rewrite(inc);
     assign(pop,sessionpathname+'population.txt');
     rewrite(pop);
     assign(clim,sessionpathname+'climate.txt');
     rewrite(clim);
     FOR i:= 1 TO NYear DO BEGIN
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         write(clim,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             WriteM(economy[j],i,els,sls,inc,pop);
         END;
         CalcClimate(i);
         write(clim,climate.temp:12:8,' ',climate.dt:12:8);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop);
         writeln(clim)
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop);
     close(clim)
END;

PROCEDURE MargRun2;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run: word;
    els,sls,inc,pop,clim: text;
    totpop,totinc,totimp,totemm,totred: double;
    hp,hy,hw: regarr;

BEGIN
     assign(els,sessionpathname+'eloss.out');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.out');
     rewrite(sls);
     assign(inc,sessionpathname+'income.out');
     rewrite(inc);
     assign(pop,sessionpathname+'population.out');
     rewrite(pop);
     assign(clim,sessionpathname+'climate.out');
     rewrite(clim);
     FOR i:= 1 TO NYear DO BEGIN
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         write(clim,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             WriteM(economy[j],i,els,sls,inc,pop);
         END;
         CalcClimate(i);
         write(clim,climate.temp:12:8,' ',climate.dt:12:8);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop);
         writeln(clim)
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop);
     close(clim)
END;

PROCEDURE ShortRep(name: string);

VAR i: word;
    outf: text;

BEGIN
     assign(outf,sessionpathname+name);
     rewrite(outf);
     WriteLn(outf,'     tax   reg    forestry  region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,tax[optper]:8:4,' ',forestry[optper]:8:2,'  ',name);
     WriteLn(outf);
     WriteLn(outf,'            NPVW        p        y        w       v    region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,NPVW:16:4,' ',population:8:4,' ',income:8:4,' ',ypc:8:4,' ',name);
     WriteLn(outf);
     WriteLn(outf,'  income 90   region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,GDP0:11:6,' ',name);
     WriteLn(outf);
     FOR i:= 28 TO NYear DO
         IF ((i+1970) MOD 25) = 0 THEN
            Writeln(outf,climate.acco2^[i]:12:6);
     close(outf)
END;

FUNCTION Objective(psnr: word): double;
(* calculates the sum total of the regional NPWs *)

VAR j: word;
    objw: double;

BEGIN
     IF ineqave = 1 THEN BEGIN
        objw:= 1;
        FOR j:= 1 TO NoReg DO
            objw:= objw*economy[j].npvw
     END
     ELSE BEGIN
          objw:= 0;
          FOR j:= 1 TO psnr DO
              objw:= objw + pow(economy[j].npvw,1-ineqave);
          objw:= objw/(1-ineqave)
     END;
     objective:= objw
END;

(* OPTIMISING PROCEDURES *)

PROCEDURE CoopCBAction(VAR T,M,N,F: measure);
(* finds the optimal emission restriction by taxing (OECD) or energy and
   carbon efficiency (non-OECD) and forestry measures under cooperation *)

VAR i, rc, mc: word;
    obj0, hobj, mobj: double;

BEGIN
     RunEconomy;
     obj0:= objective(noreg);
     mobj:= -10000000;
     FOR i:= 1 TO NoReg DO BEGIN
         objtax(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= i;
            mc:= 1
         END;
         taxobj(i);

         objch4(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= i;
            mc:= 2
         END;
         ch4obj(i);

         objn2o(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= i;
            mc:= 3
         END;
         n2oobj(i);

(*         objfor(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) THEN BEGIN
             mobj:= hobj;
             rc:= i;
             mc:= 4
         END;
         forobj(i) *)
     END;
     WHILE mobj > obj0 DO BEGIN
write('.');
           IF mc=1 THEN
              objtax(rc);
           IF mc=2 THEN
              objch4(rc);
           IF mc=3 THEN
              objn2o(rc);
(*           IF mc=4 THEN
              objfor(rc); *)
           RunEconomy;
           obj0:= objective(noreg);
           mobj:= -10000000;
           FOR i:= 1 TO NoReg DO BEGIN
               IF economy[i].tax[optper] < 0.5 THEN BEGIN
                  objtax(i);
                  RunEconomy;
                  hobj:= objective(noreg);
                  IF (hobj > mobj) THEN BEGIN
                     mobj:= hobj;
                     rc:= i;
                     mc:= 1;
                  END;
                  taxobj(i)
               END;

               IF economy[i].ch4red[optper] < 0.5 THEN BEGIN
                  objch4(i);
                  RunEconomy;
                  hobj:= objective(noreg);
                  IF (hobj > mobj) THEN BEGIN
                     mobj:= hobj;
                     rc:= i;
                     mc:= 2
                  END;
               ch4obj(i)
               END;

               IF economy[i].n2ored[optper] < 0.5 THEN BEGIN
                  objn2o(i);
                  RunEconomy;
                  hobj:= objective(noreg);
                  IF (hobj > mobj) THEN BEGIN
                     mobj:= hobj;
                     rc:= i;
                     mc:= 3
                  END;
               n2oobj(i)
               END;

(*               objfor(i);
               RunEconomy;
               hobj:= objective(noreg);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 3;
               END;
               forobj(i) *)
           END
     END;
     FOR i:= 1 TO NoReg DO BEGIN
         T[i]:= economy[i].tax[optper];
         M[i]:= economy[i].ch4red[optper];
         N[i]:= economy[i].n2ored[optper];
         F[i]:= 0 (*economy[i].forestry[optper]*)
     END
END;

PROCEDURE CoopCEAction(psnr: word; VAR T,F: measure);

VAR i, j, rc, mc: word;
    obj0, hobj, mobj: double;

BEGIN

(*economy[2].tax[optper]:= europe[optper];*)

     RunEconomy;
     obj0:= objective(psnr);
(*write(obj0:8:4,' ');*)
     mobj:= -10000000;

         (*objtax(1);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 1;
            mc:= 1;
         END;
         taxobj(1);*)

     FOR i:= 1 TO psnr DO BEGIN
(*IF emmrec[i] > 0.9*ncemmres[i,optper] THEN BEGIN*)
         objtax(i);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= i;
            mc:= 1;
         END;
         taxobj(i);
(*END;*)
(*         objfor(i);
         RunEconomy;
         hobj:= objective;
         IF (hobj > mobj) THEN BEGIN
             mobj:= hobj;
             rc:= i;
             mc:= 3;
         END;
         forobj(i) *)
     END;

(*         objtax(7);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 7;
            mc:= 1;
         END;
         taxobj(7);
         objtax(8);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 8;
            mc:= 1;
         END;
         taxobj(8);*)


     WHILE (*(mobj > obj0) OR*) globviol DO BEGIN
           (*IF mc=1 THEN*)
              objtax(rc);
           (*IF mc=3 THEN
              objfor(rc);*)
           RunEconomy;
           obj0:= objective(psnr);
(*write(obj0:8:4,' ');*)
           mobj:= -10000000;

              (* objtax(1);
               RunEconomy;
               hobj:= objective(psnr);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= 1;
                  mc:= 1;
               END;
               taxobj(1);*)

           FOR i:= 1 TO psnr DO BEGIN
(*IF emmrec[i] > 0.9*ncemmres[i,optper] THEN BEGIN*)
               objtax(i);
               RunEconomy;
               hobj:= objective(psnr);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 1;
               END;
               taxobj(i);
(*END;*)
(*               objfor(i);
               RunEconomy;
               hobj:= objective;
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 3;
               END;
               forobj(i)*)
           END;
(*         objtax(7);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 7;
            mc:= 1;
         END;
         taxobj(7);
         objtax(8);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 8;
            mc:= 1;
         END;
         taxobj(8);*)


     END;
     FOR i:= 1 TO NoReg DO BEGIN
         T[i]:= economy[i].tax[optper];
         F[i]:= 0(*economy[i].forestry[optper]*)
     END
END;

PROCEDURE NonCoopCEAction(VAR T,M,N,F: measure);

VAR i, j: word;
    emm0, tobj, mobj, nobj, fobj, obj0: double;

BEGIN
     FOR i:= 1 TO NoReg DO BEGIN
         RunEconomy;
(*writeln(ncemmres[i,optper]:8:4,' ',emmrec[i]:8:4,' ',economy[i].tax[optper]:6:4);*)
         obj0:= economy[i].npvw;
         emm0:= emmrec[i];
         objtax(i);
         (*count[i]:= 0;*)
         RunEconomy;
         tobj:= economy[i].npvw;
         taxobj(i);
         objch4(i);
         RunEconomy;
         mobj:= economy[i].npvw;
         ch4obj(i);
         objn2o(i);
         RunEconomy;
         nobj:= economy[i].npvw;
         n2oobj(i);
(*         objfor(i);
         RunEconomy;
         fobj:= economy[i].npvw;
         forobj(i);   *)
         WHILE emmrec[i] > (*1.1*)ncemmres[i,optper] (*) (count[i] > 0.5*runs) (*OR (tobj > obj0) OR (fobj > obj0)*) DO BEGIN
               IF (tobj > mobj) AND (tobj > nobj) THEN
                  objtax(i)
               ELSE IF mobj > nobj THEN
                    objch4(i)
               ELSE
                   objn2o(i);
              (* ELSE
                   objfor(i);*)
               RunEconomy;
               obj0:= economy[i].npvw;
               emm0:= emmrec[i];
               IF economy[i].tax[optper] <= 0.99 THEN BEGIN
                  objtax(i);
                  count[i]:= 0;
                  RunEconomy;
                  tobj:= economy[i].npvw;
                  taxobj(i)
               END
               ELSE
                   tobj:= -1000000;
               IF economy[i].ch4red[optper] <= 0.99 THEN BEGIN
                  objch4(i);
                  RunEconomy;
                  mobj:= economy[i].npvw;
                  ch4obj(i)
               END
               ELSE
                   mobj:= -1000000;
               IF economy[i].n2ored[optper] <= 0.99 THEN BEGIN
                  objn2o(i);
                  RunEconomy;
                  nobj:= economy[i].npvw;
                  n2oobj(i)
               END
               ELSE
                   nobj:= -1000000
            (*   objfor(i);
               RunEconomy;
               fobj:= economy[i].npvw;
               forobj(i)  *)
         END;
         T[i]:= economy[i].tax[optper];
         M[i]:= economy[i].ch4red[optper];
         N[i]:= economy[i].n2ored[optper];
         F[i]:= 0 (*economy[i].forestry[optper]*)
     END
END;

PROCEDURE CoopCBA(VAR T,M,N,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 64*btaxst;
     forstep:= 64*bforst;
     CoopCBAction(T,M,N,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].ch4red[optper] > 0 THEN
            taxobj(i);
         IF economy[i].n2ored[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     CoopCBAction(T,M,N,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].ch4red[optper] > 0 THEN
            taxobj(i);
         IF economy[i].n2ored[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     CoopCBAction(T,M,N,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].ch4red[optper] > 0 THEN
            taxobj(i);
         IF economy[i].n2ored[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     forstep:= bforst;
     CoopCBAction(T,M,N,F)
END;

PROCEDURE CoopCEA(psnr: word; VAR T,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     CoopCEAction(psnr,T,F);
     FOR i:= 1 TO psnr DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     CoopCEAction(psnr,T,F);
     FOR i:= 1 TO psnr DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     forstep:= bforst;
     CoopCEAction(psnr,T,F)
END;

PROCEDURE NonCoopCEA(VAR T,M,N,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 64*btaxst;
     ch4step:= 64*bch4st;
     n2ostep:= 64*bn2ost;
     forstep:= 64*bforst;
     NonCoopCEAction(T,M,N,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].ch4red[optper] > 0 THEN
            ch4obj(i);
         IF economy[i].n2ored[optper] > 0 THEN
            n2oobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 16*btaxst;
     ch4step:= 16*bch4st;
     n2ostep:= 16*bn2ost;
     forstep:= 16*bforst;
     NonCoopCEAction(T,M,N,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].ch4red[optper] > 0 THEN
            ch4obj(i);
         IF economy[i].n2ored[optper] > 0 THEN
            n2oobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     ch4step:= 4*bch4st;
     n2ostep:= 4*bn2ost;
     forstep:= 4*bforst;
     NonCoopCEAction(T,M,N,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].ch4red[optper] > 0 THEN
            ch4obj(i);
         IF economy[i].n2ored[optper] > 0 THEN
            n2oobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     ch4step:= bch4st;
     n2ostep:= bn2ost;
     forstep:= bforst;
     NonCoopCEAction(T,M,N,F)
END;

PROCEDURE NCOptimisation(W: integer; VAR T,M,N,F: double);
(* finds the optimal emission restriction by taxing and forestry under
   non-cooperation *)

VAR objt, objf, objm, objn, obj0: double;
    i: word;

BEGIN
write(' ',economy[w].name);

     taxstep:= 16*btaxst;
     ch4step:= 16*bch4st;
     n2ostep:= 16*bn2ost;
     forstep:= 16*bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1-coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
        (*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     taxobj(W);
     objch4(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objm:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objm:= objm + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     ch4obj(W);
     objn2o(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objn:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objn:= objn + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     n2oobj(W);
     WHILE ((objt > obj0) OR (objm > obj0) OR (objn > obj0)) DO BEGIN
           IF (objt > objm) AND (objt > objn) THEN
              objtax(W)
           ELSE IF objm > objn THEN
                objch4(W)
           ELSE
               objn2o(W);
           RunEconomy;
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
             obj0:= economy[w].npvw*(1-coop[w,optper]);
             FOR i:= 1 TO NoReg DO
                  obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           IF economy[w].tax[optper] < 0.5 THEN BEGIN
              objtax(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objt:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              taxobj(W)
           END
           ELSE
               objt:= obj0-1000;
           IF economy[w].ch4red[optper] < 0.5 THEN BEGIN
              objch4(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objm:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objm:= objm + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              ch4obj(W)
           END
           ELSE
               objm:= obj0-1000;
           IF economy[w].n2ored[optper] < 0.5 THEN BEGIN
              objn2o(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objn:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objn:= objn + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              n2oobj(W)
           END
           ELSE
               objn:= obj0-1000
     END;
     IF economy[w].tax[optper] > 0 THEN
        taxobj(w);
     IF economy[w].ch4red[optper] > 0 THEN
        ch4obj(w);
     IF economy[w].n2ored[optper] > 0 THEN
        n2oobj(w);
(*     IF economy[w].forestry[optper] > 0 THEN
        forobj(w); *)

     taxstep:= 4*btaxst;
     ch4step:= 4*bch4st;
     n2ostep:= 4*bn2ost;
     forstep:= 4*bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1-coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
        (*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     taxobj(W);
     objch4(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objm:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objm:= objm + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     ch4obj(W);
     objn2o(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objn:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objn:= objn + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     n2oobj(W);
     WHILE ((objt > obj0) OR (objm > obj0) OR (objn > obj0)) DO BEGIN
           IF (objt > objm) AND (objt > objn) THEN
              objtax(W)
           ELSE IF objm > objn THEN
                objch4(W)
           ELSE
               objn2o(W);
           RunEconomy;
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
             obj0:= economy[w].npvw*(1-coop[w,optper]);
             FOR i:= 1 TO NoReg DO
                  obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           IF economy[w].tax[optper] < 0.5 THEN BEGIN
              objtax(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objt:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              taxobj(W)
           END
           ELSE
               objt:= obj0-1000;
           IF economy[w].ch4red[optper] < 0.5 THEN BEGIN
              objch4(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objm:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objm:= objm + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              ch4obj(W)
           END
           ELSE
               objm:= obj0-1000;
           IF economy[w].n2ored[optper] < 0.5 THEN BEGIN
              objn2o(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objn:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objn:= objn + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              n2oobj(W)
           END
           ELSE
               objn:= obj0-1000
     END;
     IF economy[w].tax[optper] > 0 THEN
        taxobj(w);
     IF economy[w].ch4red[optper] > 0 THEN
        ch4obj(w);
     IF economy[w].n2ored[optper] > 0 THEN
        n2oobj(w);
(*     IF economy[w].forestry[optper] > 0 THEN
        forobj(w); *)

     taxstep:= btaxst;
     ch4step:= bch4st;
     n2ostep:= bn2ost;
     forstep:= bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1-coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
        (*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     taxobj(W);
     objch4(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objm:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objm:= objm + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     ch4obj(W);
     objn2o(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objn:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objn:= objn + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
     n2oobj(W);
     WHILE ((objt > obj0) OR (objm > obj0) OR (objn > obj0)) DO BEGIN
           IF (objt > objm) AND (objt > objn) THEN
              objtax(W)
           ELSE IF objm > objn THEN
                objch4(W)
           ELSE
               objn2o(W);
           RunEconomy;
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
             obj0:= economy[w].npvw*(1-coop[w,optper]);
             FOR i:= 1 TO NoReg DO
                  obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           IF economy[w].tax[optper] < 0.5 THEN BEGIN
              objtax(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objt:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              taxobj(W)
           END
           ELSE
               objt:= obj0-1000;
           IF economy[w].ch4red[optper] < 0.5 THEN BEGIN
              objch4(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objm:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objm:= objm + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              ch4obj(W)
           END
           ELSE
               objm:= obj0-1000;
           IF economy[w].n2ored[optper] < 0.5 THEN BEGIN
              objn2o(W);
              RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
              objn:= economy[w].npvw*(1 - coop[w,optper]);
              FOR i:= 1 TO NoReg DO
                  objn:= objn + economy[i].npvw*coop[i,optper]*coop[w,optper];
(*     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw); *)
              n2oobj(W)
           END
           ELSE
               objn:= obj0-1000
     END;

     T:= economy[w].tax[optper];
     M:= economy[w].ch4red[optper];
     N:= economy[w].n2ored[optper];
     F:= 0 (*economy[w].forestry[optper]*)
END;

PROCEDURE Optimal(T,F: measure);

var i: word;

BEGIN
     SetTax4(T);
     SetFor4(F);
     RunEconomy1;
     ShortRep('summary.out')
END;

(* EXIT PROCEDURE *)

PROCEDURE Exit;

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              dispose(ypcgrowth);
              dispose(pgrowth);
              dispose(aeei);
              dispose(acei);
              dispose(ch4em);
              dispose(n2oem);
              dispose(forestemm);
              dispose(scentemp)
         END;
     WITH Climate DO BEGIN
          dispose(addco2);
          dispose(addch4);
          dispose(addn2o);
          dispose(acco2);
          dispose(acch4);
          dispose(acn2o)
     END
END;

(* MAIN PROCEDURES *)

PROCEDURE RunNonCoopCBA;

VAR T,M,N,F: measure;
    w, cnt: integer;
    outs: text;
    stop: boolean;
    s: string;
    seet, seem, seen, seef: double;

BEGIN
     SetMeasures1;
     assign(outs,sessionpathname+'seq0.out');
     rewrite(outs);
     FOR optper:= startper TO NoPer DO BEGIN
         write(optper:2,' ');
         FOR w:= 1 TO NoReg DO BEGIN
             NCOptimisation(w,T[w],M[w],N[w],F[w]);
             economy[w].tax[optper]:= 0
         END;
         FOR w:= 1 TO NoReg DO
             write(outs,T[w]:6:4,' ');
         writeln(outs);
         FOR w:= 1 TO NoReg DO
             write(outs,M[w]:6:4,' ');
         writeln(outs);
         FOR w:= 1 TO NoReg DO
             write(outs,N[w]:6:4,' ');
         writeln(outs);
         FOR w:= 1 TO NoReg DO
             write(outs,F[w]:6:2,' ');
         writeln(outs);
     END;
     close(outs);
     cnt:= 0;
     REPEAT
           inc(cnt);
           str(cnt,s);
           assign(outs,sessionpathname+'seq'+s+'.out');
           rewrite(outs);
           SetMeasures3(cnt-1);
           FOR optper:= StartPer TO NoPer DO BEGIN
               write(optper:2,' ');
               FOR w:= 1 TO NoReg DO
                   WITH economy[w] DO BEGIN
                        seet:= tax[optper];
                        seem:= ch4red[optper];
                        seen:= n2ored[optper];
                        seef:= forestry[optper];
                        tax[optper]:= 0;
                        forestry[optper]:= 0;
                        NCOptimisation(w,T[w],M[w],N[w],F[w]);
                        tax[optper]:= seet;
                        ch4red[optper]:= seem;
                        n2ored[optper]:= seen;
                        forestry[optper]:= seef;
                   END;
               FOR w:= 1 TO NoReg DO
                   write(outs,((*0.9*economy[w].tax[optper]+0.1*)T[w]):6:4,' ');
               writeln(outs);
               FOR w:= 1 TO NoReg DO
                   write(outs,((*0.9*economy[w].tax[optper]+0.1*)M[w]):6:4,' ');
               writeln(outs);
               FOR w:= 1 TO NoReg DO
                   write(outs,((*0.9*economy[w].tax[optper]+0.1*)N[w]):6:4,' ');
               writeln(outs);
               FOR w:= 1 TO NoReg DO
                   write(outs,((*0.9*economy[w].forestry[optper]+0.1*)F[w]):6:2,' ');
               writeln(outs);
           END;
           close(outs);
           stop:= checkstop(cnt)
     UNTIL (stop OR (cnt = itmax))
END;

PROCEDURE RunNonCoopCEA;

VAR T,M,N,F: measure;
    i, cnt: word;
    s: string;
    ceaout: text;

BEGIN
     SetMeasures1;
     assign(ceaout,sessionpathname+'seq0.out');
     rewrite(ceaout);
     FOR optper:= StartPer TO NoPer DO BEGIN
         write(optper:2,' ');
         RunEconomy;
         NonCoopCEA(T,M,N,F);
         FOR i:= 1 TO NoReg DO
             write(ceaout,T[i]:6:4,' ');
         writeln(ceaout);
         FOR i:= 1 TO NoReg DO
             write(ceaout,M[i]:6:4,' ');
         writeln(ceaout);
         FOR i:= 1 TO NoReg DO
             write(ceaout,N[i]:6:4,' ');
         writeln(ceaout);
         FOR i:= 1 TO NoReg DO
             write(ceaout,F[i]:6:2,' ');
         writeln(ceaout)
     END;
     close(ceaout);
(*     cnt:= 0;
     REPEAT
           inc(cnt);
           str(cnt,s);
           assign(ceaout,sessionpathname+'seq'+s+'.out');
           rewrite(ceaout);
           SetMeasures3(cnt-1);
           FOR optper:= StartPer TO NoPer DO BEGIN
               write(optper:2,' ');
               RunEconomy;
               NonCoopCEA(T,F);
               FOR i:= 1 TO NoReg DO
                   write(ceaout,T[i]:6:4,' ');
               writeln(ceaout);
               FOR i:= 1 TO NoReg DO
                   write(ceaout,F[i]:6:2,' ');
               writeln(ceaout)
           END;
           close(ceaout);
     UNTIL checkstop(cnt)*)
END;

PROCEDURE RunCoopCBA;

VAR T,M,N,F,seet,seem,seen,seef: measure;
    i, cnt: word;
    outs: text;
    s: string;
    stop: boolean;

BEGIN
     assign(outs,sessionpathname+'seq0.out');
     rewrite(outs);
     FOR optper:= StartPer TO NoPer DO BEGIN
         SetMeasures1;
         write(optper:2,' ');
         RunEconomy;
         FOR i:= 1 TO NoReg DO
             economy[i].basewf:= economy[i].npvw;
         CoopCBA(T,M,N,F);
         FOR i:= 1 TO NoReg DO
             write(outs,T[i]:8:6,' ');
         writeln(outs);
         FOR i:= 1 TO NoReg DO
             write(outs,M[i]:8:6,' ');
         writeln(outs);
         FOR i:= 1 TO NoReg DO
             write(outs,N[i]:8:6,' ');
         writeln(outs);
         FOR i:= 1 TO NoReg DO
             write(outs,F[i]:6:2,' ');
         writeln(outs);
     END;
     close(outs);
     cnt:= 0;
     REPEAT
           inc(cnt);
           str(cnt,s);
           assign(outs,sessionpathname+'seq'+s+'.out');
           rewrite(outs);
           SetMeasures3(cnt-1);
           FOR optper:= StartPer TO NoPer DO BEGIN
               write(optper:2,' ');
               FOR i:= 1 TO NoReg DO BEGIN
                   seet[i]:= economy[i].tax[optper];
                   seem[i]:= economy[i].ch4red[optper];
                   seen[i]:= economy[i].n2ored[optper];
                   seef[i]:= economy[i].forestry[optper];
                   economy[i].tax[optper]:= 0;
                   economy[i].ch4red[optper]:= 0;
                   economy[i].n2ored[optper]:= 0;
                   economy[i].forestry[optper]:= 0
               END;
               RunEconomy;
               FOR i:= 1 TO NoReg DO
                   economy[i].basewf:= economy[i].npvw;
               CoopCBA(T,M,N,F);
               FOR i:= 1 TO NoReg DO BEGIN
                   economy[i].tax[optper]:= seet[i];
                   economy[i].ch4red[optper]:= seem[i];
                   economy[i].n2ored[optper]:= seen[i];
                   economy[i].forestry[optper]:= seef[i]
               END;
               FOR i:= 1 TO NoReg DO
                   write(outs,(0.75*economy[i].tax[optper]+0.25*T[i]):8:6,' ');
               writeln(outs);
               FOR i:= 1 TO NoReg DO
                   write(outs,(0.75*economy[i].ch4red[optper]+0.25*M[i]):8:6,' ');
               writeln(outs);
               FOR i:= 1 TO NoReg DO
                   write(outs,(0.75*economy[i].n2ored[optper]+0.25*N[i]):8:6,' ');
               writeln(outs);
               FOR i:= 1 TO NoReg DO
                   write(outs,(0.75*economy[i].forestry[optper]+0.25*F[i]):6:2,' ');
               writeln(outs);
           END;
           close(outs);
           stop:= checkstop(cnt)
     UNTIL (stop OR (cnt = itmax))
END;

PROCEDURE RunRepCBA;

VAR T,M,N,F,seet,seef,seem,seen: measure;
    i, w, cnt: word;
    outs: text;
    s: string;
    stop: boolean;
    seetd, seefd: double;


BEGIN
     assign(outs,sessionpathname+'seq0.out');
     rewrite(outs);

     FOR optper:= StartPer TO NoPer DO BEGIN
         SetMeasures1;
         write(optper:2,' ');
         RunEconomy;
         FOR i:= 1 TO NoReg DO
             economy[i].basewf:= economy[i].npvw;
         CoopCBA(T,M,N,F);
         FOR i:= 1 TO NoReg DO
             write(outs,T[i]:6:4,' ');
         writeln(outs);
         FOR i:= 1 TO NoReg DO
             write(outs,M[i]:6:4,' ');
         writeln(outs);
         FOR i:= 1 TO NoReg DO
             write(outs,N[i]:6:4,' ');
         writeln(outs);
         FOR i:= 1 TO NoReg DO
             write(outs,F[i]:6:2,' ');
         writeln(outs);
     END;

     SetMeasures1;
     FOR optper:= RepPer + 1 TO NoPer DO BEGIN
         write(optper:2,' ');
         FOR w:= 1 TO NoReg DO BEGIN
             NCOptimisation(w,T[w],M[w],N[w],F[w]);
             economy[w].tax[optper]:= 0
         END;
         FOR w:= 1 TO NoReg DO
             write(outs,T[w]:6:4,' ');
         writeln(outs);
         FOR w:= 1 TO NoReg DO
             write(outs,M[w]:6:4,' ');
         writeln(outs);
         FOR w:= 1 TO NoReg DO
             write(outs,N[w]:6:4,' ');
         writeln(outs);
         FOR w:= 1 TO NoReg DO
             write(outs,F[w]:6:2,' ');
         writeln(outs);
     END;
     close(outs);
     cnt:= 0;

     REPEAT
           inc(cnt);
           str(cnt,s);
           assign(outs,sessionpathname+'seq'+s+'.out');
           rewrite(outs);

           SetMeasures3(cnt-1);
           FOR optper:= StartPer TO RepPer DO BEGIN
               write(optper:2,' ');
               FOR i:= 1 TO NoReg DO BEGIN
                   seet[i]:= economy[i].tax[optper];
                   seef[i]:= economy[i].forestry[optper];
                   economy[i].tax[optper]:= 0;
                   economy[i].forestry[optper]:= 0
               END;
               RunEconomy;
               FOR i:= 1 TO NoReg DO
                     economy[i].basewf:= economy[i].npvw;
               CoopCBA(T,M,N,F);
               FOR i:= 1 TO NoReg DO BEGIN
                   economy[i].tax[optper]:= seet[i];
                   economy[i].ch4red[optper]:= seem[i];
                   economy[i].n2ored[optper]:= seen[i];
                   economy[i].forestry[optper]:= seef[i]
               END;
               FOR i:= 1 TO NoReg DO
                   write(outs,(*0.9*economy[i].tax[optper]+0.1*)T[i]:6:4,' ');
               writeln(outs);
               FOR i:= 1 TO NoReg DO
                   write(outs,(*0.9*economy[i].ch4red[optper]+0.1*)M[i]:6:4,' ');
               writeln(outs);
               FOR i:= 1 TO NoReg DO
                   write(outs,(*0.9*economy[i].n2ored[optper]+0.1*)N[i]:6:4,' ');
               writeln(outs);
               FOR i:= 1 TO NoReg DO
                   write(outs,(*0.9*economy[i].forestry[optper]+0.1*)F[i]:6:2,' ');
               writeln(outs);
           END;

           FOR optper:= RepPer+1 TO NoPer DO BEGIN
               write(optper:2,' ');
               FOR w:= 1 TO NoReg DO
                   WITH economy[w] DO BEGIN
                        seetd:= tax[optper];
                        seefd:= forestry[optper];
                        tax[optper]:= 0;
                        forestry[optper]:= 0;
                        NCOptimisation(w,T[w],M[w],N[w],F[w]);
                        tax[optper]:= seetd;
                        forestry[optper]:= seefd;
                   END;
                FOR w:= 1 TO NoReg DO
                    write(outs,((*0.9*economy[w].tax[optper]+0.1*)T[w]):6:4,' ');
                writeln(outs);
                FOR w:= 1 TO NoReg DO
                    write(outs,((*0.9*economy[w].tax[optper]+0.1*)M[w]):6:4,' ');
                writeln(outs);
                FOR w:= 1 TO NoReg DO
                    write(outs,((*0.9*economy[w].tax[optper]+0.1*)N[w]):6:4,' ');
                writeln(outs);
                FOR w:= 1 TO NoReg DO
                    write(outs,((*0.9*economy[w].forestry[optper]+0.1*)F[w]):6:2,' ');
                writeln(outs);
           END;
           close(outs);
           stop:= checkstop(cnt)
     UNTIL (stop OR (cnt = itmax))
END;

PROCEDURE RunCoopCEA;

VAR T,F: measure;
    i,psnr: word;
    ceaout: text;

BEGIN
     SetMeasures1;
(*     RunNonCoopCEA;*)
     assign(ceaout,sessionpathname+'cea.out');
     rewrite(ceaout);
     FOR optper:= StartPer TO NoPer DO BEGIN
         write(optper:2,' ');
         RunEconomy;
         (*IF optper < 5 THEN
            psnr:= 4
         ELSE*) psnr:= 7 (*NoReg*);
         FOR i:= 1 TO NoReg DO
             economy[i].basewf:= economy[i].npvw;
         CoopCEA(psnr,T,F);
         FOR i:= 1 TO NoReg DO
             write(ceaout,T[i]:6:4,' ');
         writeln(ceaout);
         FOR i:= 1 TO NoReg DO
             write(ceaout,F[i]:6:2,' ');
         writeln(ceaout)
     END;
     close(ceaout)
END;

PROCEDURE CalcMarg(VAR out, out2: text);

VAR els0,els1,sls0,sls1,inc0,pop0: text;
    oldinc,oldpop,newinc,newpop,elsbs,elspt,slsbs,slspt,df0,df1,df3,dfw,dfg,dfu,disc,md,npvmd: regarr;
    i,j: word;
    year,margreg0,margreg1,margreg3,margregw,margregg,margregu,margwld0,margwld1,margwld3,margwldw,margwldg,margwldu: double;
    wldpop, wldinc, greenbook: double;

(* dfw Weitzman discounting, dfg Gollier discounting, dfd UK Green Book discounting *)

BEGIN
     assign(els0,sessionpathname+'eloss.txt');
     reset(els0);
     assign(els1,sessionpathname+'eloss.out');
     reset(els1);
     assign(sls0,sessionpathname+'sloss.txt');
     reset(sls0);
     assign(sls1,sessionpathname+'sloss.out');
     reset(sls1);
     assign(inc0,sessionpathname+'income.txt');
     reset(inc0);
     assign(pop0,sessionpathname+'populati.txt');
     reset(pop0);
     read(inc0,year);
     read(pop0,year);
     read(els0,year);
     read(els1,year);
     read(sls0,year);
     read(sls1,year);
     FOR i:= 1 TO NoReg DO BEGIN
         read(inc0,oldinc[i]);
         read(pop0,oldpop[i]);
         read(els0,elsbs[i]);
         read(els1,elspt[i]);
         read(sls0,slsbs[i]);
         read(sls1,slspt[i]);
         df0[i]:= 1;
         df1[i]:= 1;
         df3[i]:= 1;
         dfw[i]:= 1;
	 dfg[i]:= 1;
	 dfu[i]:= 1;
         npvmd[i]:= 0
     END;
     margreg0:= 0;
     margreg1:= 0;
     margreg3:= 0;
     margregw:= 0;
     margregg:= 0;
     margregu:= 0;
     margwld0:= 0;
     margwld1:= 0;
     margwld3:= 0;
     margwldw:= 0;
     margwldg:= 0;
     margwldu:= 0;
     readln(inc0);
     readln(pop0);
     readln(els0);
     readln(els1);
     readln(sls0);
     readln(sls1);
     FOR i:= 2 TO NYear DO BEGIN
         read(inc0,year);
         read(pop0,year);
         read(els0,year);
         read(els1,year);
         read(sls0,year);
         read(sls1,year);
         wldpop:= 0;
         wldinc:= 0;
         FOR j:= 1 TO NoReg DO BEGIN
             read(inc0,newinc[j]);
             read(pop0,newpop[j]);
             wldpop:= wldpop + newpop[j];
             wldinc:= wldinc + newinc[j];
             read(els0,elsbs[j]);
             read(els1,elspt[j]);
             read(sls0,slsbs[j]);
             read(sls1,slspt[j])
         END;
         readln(inc0);
         readln(pop0);
         readln(els0);
         readln(els1);
         readln(sls0);
         readln(sls1);
         FOR j:= 1 TO NoReg DO BEGIN
             IF (newpop[j] < 0.00001) OR (newinc[j] < 0.00001) THEN
                disc[j]:= 0
             ELSE
                 disc[j]:= (newinc[j]/newpop[j]-oldinc[j]/oldpop[j])/(oldinc[j]/oldpop[j]);
         END;
         IF (i > 49) (*AND (i < 150)*) THEN BEGIN
            FOR j:= 1 TO NoReg DO BEGIN
                md[j]:= (elspt[j]+slspt[j]-elsbs[j]-slsbs[j])*newinc[j];
                npvmd[j]:= npvmd[j] + md[j]*df1[j];
                margreg0:= margreg0 + md[j]*df0[j];
                margreg1:= margreg1 + md[j]*df1[j];
                margreg3:= margreg3 + md[j]*df3[j];
                margregw:= margregw + md[j]*dfw[j];
                margregg:= margregg + md[j]*dfg[j];
                margregu:= margregu + md[j]*dfu[j];
                IF newinc[j] > 0.00001 THEN BEGIN
                   margwld0:= margwld0 + md[j]*df0[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld1:= margwld1 + md[j]*df1[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld3:= margwld3 + md[j]*df3[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwldw:= margwldw + md[j]*dfw[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwldg:= margwldg + md[j]*dfg[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwldu:= margwldu + md[j]*dfu[j]*wldinc*newpop[j]/wldpop/newinc[j]
                END;
                df0[j]:= df0[j]/(1.00+disc[j]);
                df1[j]:= df1[j]/(1.01+disc[j]);
                df3[j]:= df3[j]/(1.03+disc[j]);
                dfg[j]:= dfg[j]/(1.00+(0.01*((1+49*pow(2.713,-0.06*(i-50)))/( (1+7*pow(2.713,-0.06*(i-50)))))) + disc[j]);
		dfg[j]:= 1;
                dfw[j]:= dfw[j]/(1+0.375/(12.5+i-50)+disc[j]);
                IF i<80 THEN
                  greenbook:=0.035
                ELSE IF i<125 THEN
                  greenbook:=0.03
                ELSE IF i<175 THEN
                  greenbook:=0.025
                ELSE IF i<250 THEN
                  greenbook:=0.02
                ELSE IF i<350 THEN
                  greenbook:=0.015
                ELSE
                  greenbook:=0.01;
                dfu[j]:= dfu[j]/(1.00 + greenbook);

            END
         END;
         FOR j:= 1 TO NoReg DO BEGIN
             oldinc[j]:= newinc[j];
             oldpop[j]:= newpop[j]
         END
     END;
     close(els0);
     close(els1);
     close(sls0);
     close(sls1);
     close(inc0);
     close(pop0);
     write(out,margreg0:14:8,' ',margwld0:14:8,' ');
     write(out,margreg1:14:8,' ',margwld1:14:8,' ');
     write(out,margreg3:14:8,' ',margwld3:14:8,' ');
     write(out,margregw:14:8,' ',margwldw:14:8,' ');
     write(out,margregg:14:8,' ',margwldg:14:8,' ');
     writeln(out,margregu:14:8,' ',margwldu:14:8);
     FOR i:= 1 TO NoReg DO
         write(out2,npvmd[i]:14:8,' ');
     writeln(out2)
END;

PROCEDURE Main;

BEGIN
     Initialise;
     InitImpactPar;
     ReInit;
     IF CBA THEN
        IF cooperate THEN
           RunCoopCBA
        ELSE IF repeated THEN
             RunRepCBA
        ELSE
            RunNonCoopCBA
     ELSE IF CEA THEN
          IF cooperate THEN
             RunCoopCEA
          ELSE
              RunNonCoopCEA
     ELSE BEGIN
          IF Scratch THEN
             SetMeasures1
          ELSE
              SetMeasures4;
          RunEconomy1;
          ShortRep('summary.out')
     END;
     Exit
END;

PROCEDURE ShortMain;

BEGIN
     Initialise;
     InitImpactPar;
     ReInit;
     RunEconomy;
     Exit
END;

PROCEDURE MargMain;

VAR i: word;
    out, out2: text;

BEGIN
     Initialise;
     InitImpactPar;
     ReInit;
     write(' calculating');
     IF scratch THEN
        SetMeasures1
     ELSE
         SetMeasures4;
     addmp:= 0;
     MargRun1;
     addmp:= 1;
     ReInit;
     MargRun2;
     assign(out,sessionpathname+'margunc.out');
     rewrite(out);
     assign(out2,sessionpathname+'regmarg.out');
     rewrite(out2);
     CalcMarg(out,out2);
(*     FOR i:= 1 TO 6076 DO
         DrawRandom; *)
     FOR i:= 1 TO Runs DO BEGIN
         write(i:4,' ');
         DrawRandom;
         ReInit;
         addmp:= 0;
         MargRun1;
         write('run1 ');
         addmp:= 1;
         ReInit;
         MargRun2;
         writeln('run2');
         CalcMarg(out,out2)
     END;
     close(out);
     close(out2);
     Exit
END;

PROCEDURE MetaMain;

VAR i: word;
    out: text;

BEGIN
     SetMeasures1;
     assign(out,sessionpathname+'radforc.out');
     rewrite(out);
     MC:= 0;
     FOR i:= 0 TO 5000 DO BEGIN
         write(MC:6:1);
         MaxRF:= 0;
         MaxYR:= 1950;
         ShortMain;
         WriteLn(out,MC:6:1,' ',MaxRF:8:4,' ',MaxYr:4:0);
         MC:= MC+0.1
     END;
     close(out);
END;

BEGIN
     MC:= 32.5;
     Main
END.